//*********************************************************************
//
//  NAV_UPGRADE.CMD - Command script to upgrade NAV Data
//
//*********************************************************************

//! @li 2014/02/25 kKoueka.: Correction of defect 52669 (Incorrect memory space computation)
//!     correction of Defect 52716 (probress bar issue)
//!     add Map and builtins version in the beguining of log
//! @li 2014/02/12 kKoueka : Impossible to upgrade because l_quit_upg is managed the wrong way
//!     In case of intercontinental upgrade "confirm upgrade popup" has disappear
//!     and the format continue without the user answer.
//! @li 2014/01/16 kKoueka     -      : correction of Defect 52902
//! @li 2013/11/22 kKoueka  Adding Robustness due to Defect 52975.
//! @li 2013/10/10 kKoueka  Correction of Defect 49134
//!     The system restarts even if no map of the new continent has been installed


/*** UPGRADE FLAGS ***/



/*** GLOBAL DEFINES ****/

// CD types
typedef enum
{
  NAVIGATION = 0,
  LOOK_ASIDE
}t_CDType;

//Source type DVD/other
typedef enum
{
  SOURCE_NONE = -1,
  SOURCE_COUNTRY = 0,
  SOURCE_CONTINENT
}t_Source;

typedef enum
{
  MMI_UPG_DISPLAY_ASK_MOTOR_RESTART,
  MMI_UPG_DISPLAY_ASK_MOTOR,
  MMI_UPG_DISPLAY_ASK_MOTOR_STOP_GUID,
  MMI_UPG_DISPLAY_ASK_MOTOR_SUPPRESS_A_CARD,
  MMI_UPG_DISPLAY_ASK_MOTOR_SUPPRESS_THE_CARD
} UPG_QUESTIONS_INFOS;

// Continents
typedef enum
{
  UNKNOWN_CONTINENT = 0,
  EUROPE,             // 1
  AUSTRALIA,          // 2  WEST_AUSTRALIA, EAST_AUSTRALIA
  NORTH_AMERICA,      // 3  CONTINENTAL USA + CANADA + MESSICO
  SOUTH_AMERICA,      // 4  BRASILE CID 7, ARGENTINA 29
  INDIA,              // 5
  CHINA,              // 6.
  MIDDLE_EAST,        // 7  EMIRATI ARABI, OMAN, QUWAIT,
  FAR_EAST,           // 8  MACAO, HONG KONG, ....
  SOUTH_AFRICA,       // 9  SUD AFRICA CID 27
  NORTH_AFRICA,       // 10 NORD AFRICA CIDs T.B.D.
  CENTRAL_AMERICA,    // 11
  JAPAN,              // 12
  NEW_ZEALAND,        // 13 CID 31
  COMPLETE_RUSSIA=17, // 17 CID 33
  OVER_LAST_CONTINENT
} t_continent;

// Script Types
typedef int  STATUS;
typedef void DIR;
typedef int  BOOL;
typedef int  size_t;
typedef void FILE;
#define TRUE                        (1)
#define FALSE                       (0)
#define OK                          (0)
#define ERROR                       (-1)
#define ESCAPE                      (2)
#define NULL                        (void*) 0
#define O_RDWR                      (2)
#define SEEK_SET                    (0)

#define DELAY_5S                    (int) 2500

#define UPG_MAX_TRIALS              4

#define ALL_COUNTRY "000"

// Constants
#define MAX_DRIVE_NAME_LENGTH    10
#define MAX_FILENAME_LENGTH     100
#define MAX_PATH_LENGTH         100
#define MAX_VERSION_LENGTH       95
#define MAX_CRC_LENGTH           20
#define MAX_INF_LINE_LENGTH     100
#define MAX_QUESTION_LENGTH     100
#define MAX_CID_LENGTH           95
#define MAX_CID_SIZE_LENGTH      95
#define MAX_COUNTRY_NAME_LENGTH  95
#define MAX_TIME_TO_PRINT_LENGTH 20
#define CMD_MEDIA_TYPE_MAX_SIZE     50

#define MAX_STRING_LEN          200

#define ESTIMATED_TRANFER_RATE  200000
#define FREE_SPACE_RATIO        95

// Drive letters
#define DRIVE_CDROM "/D"
#define DRIVE_HDD   "/SDC"
#define DRIVE_NAV   "/SDH"
#define DRIVE_TFFS  "/F"
#define DRIVE_USB   "/bd0"

// Key Files
#define KEY_FILE_NAV              "/CD_VER.NAV"
#define KEY_FILE_LA               "/CD_VER.LA"
#define DVD_KEY_FILE_NAV          "/DVD_VER.NAV"
#define UPG_DB_DWNL_PPC           "/db_dwnl_ppc.out"
#define UPG_DB_DWNL_PPC_INF       "/db_dwnl_ppc.out.inf"
#define UPG_DB_DWNL_PPC_TEMP      "/TEMP_db_dwnl_ppc.out"
#define UPG_DB_DWNL_PPC_INF_TEMP  "/TEMP_db_dwnl_ppc.out.inf"
#define UPG_GRUPPO_DAT            "/GRUPPO_4_ROOT.DAT"
#define UPG_GRUPPO_DAT_INF        "/GRUPPO_4_ROOT.DAT.inf"
#define UPG_GRUPPO_DAT_TEMP       "/TEMP_GRUPPO_4_ROOT.DAT"
#define UPG_GRUPPO_DAT_INF_TEMP   "/TEMP_GRUPPO_4_ROOT.DAT.inf"

#define BUILTINS_FILE_NORMAL    "/UPG/builtinsRNEG.out"
#define UPG_MAP                 "/MAP"

#define UPG_RNEG2010_EUROPE       "RNEG2010_EUROPE"
#define UPG_KEY_WORD              "SUBVER"



//Key Files UNIQUE CARTO
#define UPG_MAP_MAIN_DESCRI_DAT_FILE    "MAPPE/DESCRI.DAT"

//Key File for Size computation
#define UPG_EPC_INF_SURFIXE       "_EPC.BIN.inf"

//Key name for map root dir
#define UPG_MAPPE_DIR             "MAPPE/"
#define UPG_PATH_DELIMITER        "/"

#define UPG_INF_XTENSION          ".INF"

// Key Directories
#define TEMP_DB_ZONE          "/SDI/TEMP_DB_ZONE"

/*** SOFTWARE UPGRADE DEFINES ***/

typedef struct
{
  char  m_pFileName[MAX_PATH_LENGTH];
  char  m_pTargetPath[MAX_PATH_LENGTH];
  int   m_iSize;
  char  m_pCID[MAX_CID_LENGTH+1];
} t_NavUpgData;

//HDD country states compared to CD country states
typedef enum
{
  NEWER = 0 ,
  OLDER,
  UP_TO_DATE,
  TO_UPDATE
} UPG_CID_STATE;

typedef struct
{
  UPG_CID_STATE m_state;
  char m_pCID[MAX_CID_LENGTH+1];
  char m_pCDVersion[MAX_VERSION_LENGTH+1];
  char m_pCDRelease[MAX_VERSION_LENGTH+1];
  char m_pHDDVersion[MAX_VERSION_LENGTH+1];
  char m_pHDDRelease[MAX_VERSION_LENGTH+1];
  char m_CID_name[MAX_VERSION_LENGTH+1];
  unsigned long m_cid_size;
} t_CID_List;

#define MAX_DATA_PROVIDER 20
#define MAX_COUNTRY_NAME_HDD 30
#define UPG_MAX_PROVIDER 15


typedef struct
{
  char        m_DVD_title[MAX_VERSION_LENGTH];
  int         m_DVD_CID_number;
  t_CID_List *m_CID_list;
} t_DVD_Content;

typedef enum
{
  DVD_DEV,
  CD_DEV,
  UNKNOWN_DEV
} t_TYPE_DEVICE;

// WARNING : This structure and the one of the external_upgrade.h must be the sames.
#define MAX_COUNTRY_NAME_HDD      30
typedef struct t_UPGCIDSize
{
  unsigned short  CID;
  char            pCID[MAX_COUNTRY_NAME_HDD];
  char            CountryName[MAX_COUNTRY_NAME_HDD];
  char            Quarter;
  int             Year;
  unsigned long   CIDSize;
} t_UPGCIDSize;


/*** EXTERN FUNCTION DECLARATIONS ***/

// File operations
int    logMsg (char*, int, int, int, int, int, int);
DIR*   opendir (char * dirName);
int    open (const char * name, int flags, int mode);
STATUS close (int fd);
STATUS closedir (DIR *dir);
STATUS xdelete (const char * source);
STATUS GetTargetPath (char *FileName, char *TargetDrive, char *TargetPath);
STATUS GetFileSize (char *file, int *size);
char*  fgets (char * buf, size_t n, FILE * fp);
int    fputs (char * s, FILE * fp);
FILE*  fopen (char * file, char * mode);
int    fclose (FILE * fp);
int    feof (FILE * fp);
int    fseek (FILE * fp, long offset, int whence);

// String operations
STATUS strcpy (char* target, char* source);
STATUS strncpy (char* target, char* source, int n);
STATUS strcat (char* target, char* source);
int    strcmp (char* str1, char* str2);
int    strncmp (char* str1, char* str2, int n);
STATUS strlen (char* str);
char*  strstr (char* str, char* substr);

// Others
unsigned long tickGet (void);
STATUS taskDelay (int ticks);
void*  malloc (size_t nBytes);
void*  realloc (void *pBlock, size_t newSize);
void   free (void * ptr);
int    atoi (char * p_str);
long   atol (char * p_str);


// MMI UPG Panel management
STATUS ClearScreen (void);
STATUS ShowNavigationBargraph (void);
STATUS StepBargraph (char *text, int index);
STATUS ShowCDVerification (void);
STATUS ShowCDIdentification (void);
STATUS ShowCDProblem (void);
STATUS ShowNavigationQuestionScreen (char *text, int *answer); // For compatibility before CD 6028
STATUS ShowNavigationExtendedQuestionScreen (int p_InfoIndex, char *p_Text, int *p_Answer);
STATUS ShowNavigationCountryScreen (t_DVD_Content* p_nav_content, char* p_name_choice, char* p_CID_choice, int* answer);
STATUS SendMessageMMINavCDLoaded (int p_media_type);
STATUS GetMessageMMINavCIDLoaded (void);

// Builtins
STATUS ShowNavigationErasingCountryScreen (t_DVD_Content* p_nav_content, char* p_name_choice, char* p_CID_choice, int* answer, unsigned long p_size_to_install);
STATUS ShowCDCanceled (void);
STATUS AddUPGHistoryCommentary (char *p_Commentary);
STATUS UnCompress (char *p_tar_file, char *p_dest_folder);
STATUS ActivateBargraph (char *p_test, unsigned long p_TotalSize, unsigned long p_CompletedSize);
STATUS DeactivateBargraph (void);
int    GetSDBlockSize (void);

//Builtins integration for unique carto
STATUS UpdateDataForCartoTree (char* p_media_drive,
                          char* p_product_drive,
                          int* p_media_drive_size);
STATUS GetNavUpgType(char* p_descri_dat_file,
                   int* p_descri_file_lengh,
                   char* p_abslute_path,
                   int* p_upg_type);
STATUS GetUpgDVD_CurrVersNaviFile (char* p_currVersNaviFile, char* p_path);
STATUS Get_DVD_CIDInfoList(t_CID_List **l_CID_List);

// Obsolete, only for RNEG 2.0.C
STATUS ShowNavigationEraseCountryScreen (t_DVD_Content* p_nav_content, char* p_name_choice, char* p_CID_choice, int* answer);

// BCM UPG services
STATUS GetNavigationValue (char *KeyFile, char *field, char* value);
STATUS UPGCopy (char* p_source, char* p_dest, int p_cut);
STATUS UPGDelete (char* p_file);
STATUS UPGMKDir (char* p_dir);
STATUS UPGMKDelete (char* p_dir);
int    UPGSearchFile (char *Filename, char *RootDir, char *Param, char *TableDir);
STATUS UPGGetSearchResult (int index , char *SearchResult);
STATUS AddUPGHistoryHeader (char *p_CDNum, char *p_TargetDrive, char *p_Action);
STATUS SetUpgradeLogging (int TRUEFALSE);
STATUS EnableHistoryHandler (int TRUEFALSE);
STATUS UPGLogMsg (char* str, int p1, int p2, int p3, int p4, int p5, int p6);
int    IsUpgCheatCodeActivated (void);
STATUS Format (char * p_partition);
STATUS RebootSystem (void);
STATUS GetDVDTitle (char* p_file_name, char* p_dvd_name);
STATUS UnprotectHddPartition (const char* p_part_name);
STATUS GetFileVersion (char *l_pBynaryFileName, char *l_pVersion);
STATUS CheckCRCFile (char *l_file);
STATUS GetInfValue (char *File, char *field, char* value);
STATUS UPGGetFreeSpaceOnPartition (char *p_pPart, unsigned long *p_pSpaceInBytes);
STATUS UPGGetTotalSpaceOnPartition (char *p_pPart, unsigned long *p_pSpaceInBytes);
STATUS UPGGetHddCountries (int *p_pFoundCountries, t_UPGCIDSize **p_pCountries);
STATUS UPGStopGuidance (void);
STATUS UPGCheckCarto (void);


/*** INTERNAL FUNCTIONS DECLARATIONS ***/
STATUS      GetContinentOnMedia (char *p_pPath, t_continent *p_pCD_Continent);
STATUS      ComputeNavTargetPath (char *p_pTargetPath, char *p_pSourcePath, char *p_pCID, char *p_pSourceDrive, char *p_pTargetDrive);
int         FileExists (char *p_pPath);
STATUS      GetFileName (char *p_pPath, char *p_pFileName);
int         IsOnRoot (char *p_pPath);
STATUS      ReplaceInPath (char *p_pPath, char *p_pStringToReplace, char*p_pStringToReplaceWhith);
void        CleanTemporaryFiles (char *p_pTargetDrive, char *p_pCID);
t_continent GetContinent (int p_cid);
t_continent GetInstalledContinent (void);
STATUS      ExtractFileInfos (char *p_pSourceDrive, char* p_filename, t_DVD_Content* p_dvd_content);
STATUS      CheckSourceIntegrity (char *p_pSourceDrive);
STATUS      GetInfosOnMedia (t_Source l_MediaType, char *p_pSourceDrive, t_DVD_Content *l_DVD_Content, int *l_CID_Count, t_CID_List **l_CID_List);
STATUS      GetCIDAndVersionDVDOnMedia  (t_CID_List *p_CID_List, char *p_pCID, char *p_pCDVerHigh, char *p_pCDVerLow);
STATUS      GetVersionCDOnMedia (char *p_pSourceDrive,
                              char *p_pCID,
                              char *p_pKeyFileName,
                              char *p_pCDVerHigh,
                              char *p_pCDVerLow);
void        GetVersionOnProduct (char *p_pTargetDrive, char *p_pCID, char *p_pRT3VerHigh, char *p_pRT3VerLow);
BOOL        CheckProductVsCDMediaVersions (char *p_pRTVerHigh, char *p_pRTVerLow, char *p_pCDVerHigh, char *p_pCDVerLow);
UPG_CID_STATE CheckProductVsDVDMediaVersions (char *p_pRT3VerHigh, char *p_pRT3VerLow, char *p_pCDVerHigh, char *p_pCDVerLow);
int         DVDMakeCountriesListToUpgrade (t_DVD_Content *p_pDVD_Content, int p_count_real_list, t_CID_List *l_CID_real_List);
STATUS      DVDCheckSpaceAndConfirmUpgrade (char *p_pSourceDrive, char *p_pTargetDrive, char *p_user_country_CID_choice, char *p_user_country_name_choice, int p_count_real_list, BOOL p_bUpgFormatRequired, t_DVD_Content *p_reduced_DVD_Content, BOOL *p_script_status, t_TYPE_DEVICE  l_MediaInfoType);
STATUS      CDCheckSpaceAndConfirmUpgrade (char *p_pSourceDrive,
                                       char* p_path_Data,
                                       char *p_pTargetDrive,
                                       char *p_pKeyFileName,
                                       char *p_pCDVerHigh,
                                       char *p_pCDVerLow,
                                       BOOL *p_pbUpgRequired,
                                       t_TYPE_DEVICE
                                       p_MediaInfoType,
                                       char *p_user_country_name_choice);
void        RemoveUnwantedFiles (void);
BOOL        DVDGetCIDFromReducedList (t_DVD_Content *p_pReduced_DVD_Content, int p_i, char *p_pCID);
STATUS      CopyFileFromSrcDriveToTargDrive (char *p_pPathSourceFile, char *p_pPathTargetFile, char *p_pSourceDrive, char *p_pTargetDrive);
STATUS      FormatPartNavigation (char *p_pSourceDrive, char *p_pTargetDrive);
BOOL        CheckAndSaveFileIfNeeded (char *p_pFilePath, char *p_pSourceDrive, char *p_pTargetDrive);
STATUS      PrepareFilesListToCopy (char *p_pSourceDrive, char *p_pTargetDrive, char *p_pCID, t_NavUpgData **p_pList, int *p_piCountList, unsigned long *p_plTotalSize);
void        UPGXdeleteCountry (char *p_pTargetDrive, char *p_user_country_CID_choice);
int         UPGSizeManagement (char *p_pSourceDrive, char *p_pTargetDrive, unsigned long p_size_to_install, BOOL *p_pbUpgRequired, t_TYPE_DEVICE p_MediaInfoType);
STATUS      UPGInformNavForNewMapAndWaitAck (t_TYPE_DEVICE  l_MediaInfoType);
BOOL        UPGCheckIfVersionIsNewerThan (char *p_pSourcePath, char *p_pTargetPath);
STATUS      GetCIDSize (char *p_pPath, char *p_psCID_Size,unsigned long *p_plCID_Size);
STATUS      GetCID_EPCSize (char *p_pPath, char *p_psCID_Size,unsigned long *p_plCID_Size);
STATUS      GetConfigSystemVersion(unsigned char *p_pVersion);
STATUS      UpgIsGruppoValid(char *p_filename, char* p_filepath, char *p_CID);
int         CanUpgContinue (char* p_src_drive,
                         int p_quit_asked,
                         STATUS l_progress_status);
STATUS UncompressBinWithTriesIfError (char* p_Path_bin_file, char* p_dest_folder);

STATUS GetCorrectCountry_Size(char* p_path_dir,
                         char* p_cid,
                         unsigned long *p_total_size);

STATUS      LogBuiltinsAndMapVersion( char*   p_root);

/****************************** GLOBALS ******************************/


/****************************** MAIN ******************************/

int main (int argc, char **argv)
{
  // Local variables for general purpose
  STATUS l_script_status = ERROR;

  //first used in INITIALISATION SCRIPT context
  char l_pSourceDrive[MAX_DRIVE_NAME_LENGTH];
  char l_pTargetDrive[MAX_DRIVE_NAME_LENGTH];
  char l_pKeyFileName[MAX_FILENAME_LENGTH];
  int  l_media_drive_size;
  int l_descri_name_length;
  int l_upg_type;
  t_continent   l_CD_Continent;
  t_Source      l_MediaType;
  t_TYPE_DEVICE l_MediaInfoType;
  t_DVD_Content l_DVD_Content;

  //First used in CHECK COMPATIBILITY
  char l_ConfigSystemVersion[MAX_VERSION_LENGTH];
  char l_path_Data[CMD_MEDIA_TYPE_MAX_SIZE];
  t_continent   l_current_continent;
  BOOL l_bUpgFormatRequired;
  BOOL l_MustRebootAfterUpgrade;
  int  l_CID_Count;
  t_CID_List* l_CID_List;


  char l_pSourcePath [MAX_PATH_LENGTH];
  char l_pTargetPath [MAX_PATH_LENGTH];
  char l_Path_dest_bin_file[MAX_PATH_LENGTH];

  int  l_i = 0;

  char l_pPath[MAX_PATH_LENGTH];

  char l_pCID[MAX_CID_LENGTH];
  char l_pCIDSize[MAX_CID_LENGTH];
  unsigned long  l_lCIDSize;

  char l_pCDVerHigh[MAX_INF_LINE_LENGTH];
  char l_pCDVerLow[MAX_INF_LINE_LENGTH];
  char l_pCDVerHighTemp[MAX_INF_LINE_LENGTH];
  char l_pCDVerLowTemp[MAX_INF_LINE_LENGTH];

  char l_pRT3VerHigh[MAX_INF_LINE_LENGTH];
  char l_pRT3VerLow[MAX_INF_LINE_LENGTH];

  char l_Path_bin_file[MAX_PATH_LENGTH];
  char l_Path_gruppo_file[MAX_PATH_LENGTH];
  char l_dest_folder[MAX_PATH_LENGTH];

  BOOL l_quit_upg                   = FALSE;
  BOOL l_bUpgRequired               = FALSE;
  int  l_iCountList                 = 0;
  char l_user_country_name_choice[MAX_COUNTRY_NAME_LENGTH];
  char l_user_country_CID_choice[MAX_CID_LENGTH];
  char l_string[MAX_STRING_LEN];

  t_NavUpgData *l_pList             = NULL;
  unsigned long l_lTotalSize        = 0;
  unsigned long l_lCompletedSize    = 0;


  char l_actual_CID[MAX_CID_LENGTH+1] ;

  int  l_NumberOfCountriesToUpgrade = 0;
  t_CID_List *l_CID_real_List = NULL;
  t_DVD_Content l_reduced_DVD_Content ;

  BOOL l_bCIDtoUpdate = FALSE;
  int  l_nb_trials    = UPG_MAX_TRIALS;



  /** Variable initialization **/
  l_DVD_Content.m_CID_list = NULL;
  l_DVD_Content.m_DVD_CID_number = 0;
  l_reduced_DVD_Content.m_CID_list = NULL;
  l_reduced_DVD_Content.m_DVD_CID_number = 0;
  l_user_country_name_choice[0] = '\0';
  l_user_country_CID_choice[0] = '\0';
  l_media_drive_size = -1;
  l_CD_Continent      = UNKNOWN_CONTINENT;
  l_MediaType         = SOURCE_NONE;
  l_MediaInfoType     = UNKNOWN_DEV;
  l_current_continent = UNKNOWN_CONTINENT;
  //check compatibility
  l_bUpgFormatRequired         = FALSE;
  l_MustRebootAfterUpgrade = FALSE;
  l_CID_Count = 0;
  l_CID_List = NULL;

  /*** SCRIPT INITIALIZATION ***/
  SetUpgradeLogging (TRUE);
  EnableHistoryHandler (TRUE);

  //remove of temp file  if failure
  UPGLogMsg ("Delete all \"/SDH/TEMP_*\" files\n", 0, 0, 0, 0, 0, 0);
  xdelete ("/SDH/TEMP_*");

  UPGLogMsg ("************* NAV_UPGRADE_SUB_RNEG.CMD *******************\n", 0, 0, 0, 0, 0, 0);

  //Get current target drive "/H"
  strcpy (l_pTargetDrive, DRIVE_NAV);

  //Get current source drive
  UPGLogMsg ("(NAV_UPGRADE_SUB_RNEG.CMD)  argv[1] is '%s'\n", (int) argv[1], 0, 0, 0, 0, 0);
  strcpy (l_pSourceDrive, argv[1]);
  UPGLogMsg ("(NAV_UPGRADE_SUB_RNEG.CMD)  l_pSourceDrive is '%s'\n", (int) l_pSourceDrive, 0, 0, 0, 0, 0);

  if (ERROR == LogBuiltinsAndMapVersion (l_pSourceDrive))
  {
    UPGLogMsg ("[ERROR] Calling LogBuiltinsAndMapVersion() Failed!! \n",
               0, 0,0,0,0,0 );
  }

  UPGLogMsg ("****************************************************************\n", 0, 0, 0, 0, 0, 0);

  // Script initialization
  UPGLogMsg ("****************************************************************\n", 0, 0, 0, 0, 0, 0);
  UPGLogMsg ("*******(NAV_UPGRADE_SUB_RNEG.CMD)  Script initialization \n", 0, 0, 0, 0, 0, 0);
  UPGLogMsg ("*******(NAV_UPGRADE_SUB_RNEG.CMD)  Time = %ld\n", tickGet(), 0, 0, 0, 0, 0);
  UPGLogMsg ("****************************************************************\n", 0, 0, 0, 0, 0, 0);
  l_script_status = OK;

  if (TRUE == CanUpgContinue(l_pSourceDrive, l_quit_upg, l_script_status))
  {
    //update carto tree data
    l_media_drive_size = strlen(l_pSourceDrive);
    UPGLogMsg ("(NAV_UPGRADE_SUB_RNEG.CMD)  l_media_drive_size is '%d' \n", (int) l_media_drive_size, 0, 0, 0, 0, 0);
    l_script_status = UpdateDataForCartoTree (l_pSourceDrive,
                                              DRIVE_NAV,
                                              &l_media_drive_size);
    if (ERROR == l_script_status)
    {
      UPGLogMsg ("(NAV_UPGRADE_SUB_RNEG.CMD)  UpdateDataForCartoTree() FAILED \n", 0, 0, 0, 0, 0, 0);
    }
  }
/*
  else
  {
    if (TRUE == FileExists(l_pSourceDrive))
    {
      UPGLogMsg ("(NAV_UPGRADE_SUB_RNEG.CMD)  Source file exist \n", 0, 0, 0, 0, 0, 0);
    }

    if (FALSE == l_quit_upg)
    {
      UPGLogMsg ("(NAV_UPGRADE_SUB_RNEG.CMD)  Do not leave yet \n", 0, 0, 0, 0, 0, 0);
    }

    if (OK == l_script_status)
    {
      UPGLogMsg ("(NAV_UPGRADE_SUB_RNEG.CMD)  can continue \n", 0, 0, 0, 0, 0, 0);
    }
  }
*/

  //check for type of upgrade
  if (TRUE == CanUpgContinue(l_pSourceDrive, l_quit_upg, l_script_status))
  {

    //get upgrade type.
    l_descri_name_length = strlen(UPG_MAP_MAIN_DESCRI_DAT_FILE);
    l_script_status = GetNavUpgType (UPG_MAP_MAIN_DESCRI_DAT_FILE,
                                     &l_descri_name_length,
                                     l_pKeyFileName,
                                     &l_upg_type);
    if (ERROR == l_script_status)
    {
      UPGLogMsg ("(NAV_UPGRADE_SUB_RNEG.CMD)  GetNavUpgType() FAILED \n", 0, 0, 0, 0, 0, 0);
    }
  }

  //check to retreive CONTINENT_ID
  if (TRUE == CanUpgContinue(l_pSourceDrive, l_quit_upg, l_script_status))
  {
    UPGLogMsg ("(NAV_UPGRADE_SUB_RNEG.CMD)  l_pKeyFileName = '%s' \n upgrdeType = '%d'",
               (int) l_pKeyFileName,
               l_upg_type, 0, 0, 0, 0);
    //Get CONTINENT_ID
    l_script_status = GetContinentOnMedia (l_pKeyFileName, &l_CD_Continent);
    if (ERROR == l_script_status)
    {
      UPGLogMsg ("(NAV_UPGRADE_SUB_RNEG.CMD)  GetContinentOnMedia() FAILED \n", 0, 0, 0, 0, 0, 0);
    }
  }

  if (TRUE == CanUpgContinue(l_pSourceDrive, l_quit_upg, l_script_status))
  {
    UPGLogMsg ("(NAV_UPGRADE_SUB_RNEG.CMD)  l_CD_Continent = '%d'\n", (int) l_CD_Continent, 0, 0, 0, 0, 0);

    //Get key file
    switch (l_upg_type)
    {
      case SOURCE_COUNTRY :
      {
        l_MediaType     = SOURCE_COUNTRY;
        l_MediaInfoType = CD_DEV;
        AddUPGHistoryHeader ("   ", l_pSourceDrive, "NAV CD");
      }
      break;
      case SOURCE_CONTINENT :
      {
        l_MediaType     = SOURCE_CONTINENT;
        l_MediaInfoType = DVD_DEV;

        l_script_status = GetDVDTitle (l_pKeyFileName, l_DVD_Content.m_DVD_title);
        AddUPGHistoryHeader (l_DVD_Content.m_DVD_title, l_pSourceDrive, "NAV DVD");
        //UPGLogMsg ("(NAV_UPGRADE_SUB_RNEG.CMD)  DVD_title '%s'\n",
        //           (int)l_DVD_Content.m_DVD_title, 0, 0, 0, 0, 0);
      }
      break;
      default :
      {
        UPGLogMsg ("(NAV_UPGRADE_SUB_RNEG.CMD)   INVALID ARGUMENT : Upgrade type is '%s'\n", (int) l_pKeyFileName, 0, 0, 0, 0, 0);
        l_MediaType     = SOURCE_NONE;
        l_script_status = ERROR;
      }break;
    }
  }

  if (TRUE == CanUpgContinue(l_pSourceDrive, l_quit_upg, l_script_status))
  {
    UPGLogMsg ("(NAV_UPGRADE_SUB_RNEG.CMD)  Target drive is '%s'\n", (int) l_pTargetDrive, 0, 0, 0, 0, 0);
    UPGLogMsg ("(NAV_UPGRADE_SUB_RNEG.CMD)  Source key file is '%s'\n", (int) l_pKeyFileName, 0, 0, 0, 0, 0);

    UPGLogMsg ("****************************************************************\n", 0, 0, 0, 0, 0, 0);
    UPGLogMsg ("*******(NAV_UPGRADE_SUB_RNEG.CMD)  Script initialization \n", 0, 0, 0, 0, 0, 0);
    UPGLogMsg ("*******(NAV_UPGRADE_SUB_RNEG.CMD)  Time = %ld\n", tickGet(), 0, 0, 0, 0, 0);
    UPGLogMsg ("****************************************************************\n", 0, 0, 0, 0, 0, 0);
  }

  if (TRUE == CanUpgContinue(l_pSourceDrive, l_quit_upg, l_script_status))
  {
    UPGLogMsg ("****************************************************************\n", 0, 0, 0, 0, 0, 0);
    UPGLogMsg ("*******(NAV_UPGRADE_SUB_RNEG.CMD)  Check Compatibility \n", 0, 0, 0, 0, 0, 0);
    UPGLogMsg ("*******(NAV_UPGRADE_SUB_RNEG.CMD)  Time = %ld\n", tickGet(), 0, 0, 0, 0, 0);
    UPGLogMsg ("****************************************************************\n", 0, 0, 0, 0, 0, 0);

    l_script_status = GetConfigSystemVersion((unsigned char*)l_ConfigSystemVersion);
    if (ERROR == l_script_status)
    {
      UPGLogMsg ("(NAV_UPGRADE_SUB_RNEG.CMD)  GetConfigSystemVersion FAILED\n", 0, 0, 0, 0, 0, 0);
    }
  }

  //*** CHECK CONFIG SYSTEM VERSION ***
  if (TRUE == CanUpgContinue(l_pSourceDrive, l_quit_upg, l_script_status))
  {
    if( 0 == strcmp (l_ConfigSystemVersion, UPG_RNEG2010_EUROPE))
    {
      // MAP Upgrade is not authorised for RNEG 2010 EUROPE
      l_script_status = ERROR;
      UPGLogMsg ( "(NAV_UPGRADE_SUB_RNEG.CMD)   MAP Upgrade is not authorised for RNEG 2010 EUROPE", 0,0,0,0,0,0);
    }
  }

  if (TRUE == CanUpgContinue(l_pSourceDrive, l_quit_upg, l_script_status))
  {
    UPGLogMsg ("(NAV_UPGRADE_SUB_RNEG.CMD)   Version check +, Time = %ld\n", tickGet(), 0, 0, 0, 0, 0);
    ShowCDIdentification();
    taskDelay (100);

    l_current_continent = GetInstalledContinent();
    UPGLogMsg ("(NAV_UPGRADE_SUB_RNEG.CMD)   l_current_continent = %d\n",
               (int) l_current_continent, 0, 0, 0, 0, 0);

    //If continent change, format partition
    if (l_current_continent != l_CD_Continent)
    {
      UPGLogMsg ("(NAV_UPGRADE_SUB_RNEG.CMD)   ", 0, 0, 0, 0, 0, 0);
      UPGLogMsg (" l_current_continent (%d) != l_CD_Continent (%d) : SDH must be FORMATED!\n",
                 (int)l_current_continent,
                 (int)l_CD_Continent, 0, 0, 0, 0);
      l_bUpgFormatRequired     = TRUE;
      l_MustRebootAfterUpgrade = FALSE;
    }

    // If the continent on the media can't be identified...
    if (UNKNOWN_CONTINENT == l_CD_Continent)
    {
      UPGLogMsg ("(NAV_UPGRADE_SUB_RNEG.CMD)   Cannot find CD's continent\n", 0, 0, 0, 0, 0, 0);
      l_script_status = ERROR;
    }

  }


  if (TRUE == CanUpgContinue(l_pSourceDrive, l_quit_upg, l_script_status))
  {
    // Count CIDs, Get Infos on media
    l_script_status = GetInfosOnMedia ( l_MediaType,
                                       l_path_Data,
                                      &l_DVD_Content,
                                      &l_CID_Count,
                                      &l_CID_List);

    if (ERROR == l_script_status)
    {
      UPGLogMsg ("(NAV_UPGRADE_SUB_RNEG.CMD)  GetInfosOnMedia FAILED\n", 0, 0, 0, 0, 0, 0);
    }
  }

  if (TRUE == CanUpgContinue(l_pSourceDrive, l_quit_upg, l_script_status))
  {
    if (ERROR == CheckSourceIntegrity (l_pSourceDrive))
    {
      UPGLogMsg ("CheckSpaceAndConfirmUpgrade: CheckSourceIntegrity Error\n", 0, 0, 0, 0, 0, 0);
      l_script_status = ERROR;
    }
  }

  UPGLogMsg ("Version check -, Time = %ld\n", tickGet(), 0, 0, 0, 0, 0);

  // Here : - l_pSourceDrive
  //        - l_DVD_Content contain all the DVD informations
  //        - l_NumberOfCountriesToUpgrade contain the number of updatable countries on the DVD (1 if CD)
  //        - l_CID_real_List constain the list of updatable countries on the DVD
  //        - l_pCID contain the CID present on CD
  //        - l_current_continent contain the current continent on HDD (on CD)
  l_NumberOfCountriesToUpgrade = 1;
  RemoveUnwantedFiles ();


  while ( (TRUE  == CanUpgContinue(l_pSourceDrive, l_quit_upg, l_script_status))
          && (l_NumberOfCountriesToUpgrade > 0)
          && (FALSE == l_quit_upg))
  {
    if (TRUE == CanUpgContinue(l_pSourceDrive, l_quit_upg, l_script_status))
    {
      // CD type media ?
      if (SOURCE_COUNTRY == l_MediaType)
      {
        UPGLogMsg ("l_MediaType = SOURCE_COUNTRY\n", 0, 0, 0, 0, 0, 0);

        l_script_status = GetVersionCDOnMedia  (l_path_Data,
                                                l_pCID,
                                                l_pKeyFileName,
                                                l_pCDVerHigh,
                                                l_pCDVerLow);
        if (l_CD_Continent == l_current_continent)
        {
          GetVersionOnProduct (l_pTargetDrive,
                               l_pCID,
                               l_pRT3VerHigh, l_pRT3VerLow);

          l_bUpgRequired = CheckProductVsCDMediaVersions (l_pRT3VerHigh,
                                                          l_pRT3VerLow,
                                                          l_pCDVerHigh,
                                                          l_pCDVerLow);
          if (FALSE == l_bUpgRequired)
          {
            strcpy (l_string, "UPGRADE CANCELED BY A MEDIA VERSION OLDER THAN THE PRODUCT!\n");
            UPGLogMsg ("%s", l_string, 0, 0, 0, 0, 0);
            AddUPGHistoryCommentary (l_string);
            l_NumberOfCountriesToUpgrade = 0;
          }
          else
          {
            if (0 == strcmp ("024",l_pCID ))
            {
              if ( (0 == strcmp(l_pRT3VerHigh, "0")) && (0 == strcmp(l_pRT3VerLow, "0")) )
              {
                strcpy (l_string, "\n\n format required\n");
                UPGLogMsg ("%s", l_string, 0, 0, 0, 0, 0);
                l_bUpgFormatRequired = TRUE;
                l_MustRebootAfterUpgrade = TRUE;
              }
            }
            l_NumberOfCountriesToUpgrade = 1;
            logMsg ("l_NumberOfCountriesToUpgrade = %d\n", l_NumberOfCountriesToUpgrade, 0, 0, 0, 0, 0);
          }
        }
        else
        {
          UPGLogMsg ("l_current_continent (%d) != l_CD_Continent (%d) : Upgrade required!!!!\n",
                      (int)l_current_continent, (int)l_CD_Continent, 0, 0, 0, 0);
          l_bUpgRequired = TRUE;
        }
      }
      // DVD type media !
      else
      {
        UPGLogMsg ("l_MediaType = SOURCE_CONTINENT\n", 0, 0, 0, 0, 0, 0);

        l_NumberOfCountriesToUpgrade = 0;
        for (l_i=0 ; l_i<l_CID_Count ;l_i++)
        {
          l_script_status = GetCIDAndVersionDVDOnMedia (&(l_CID_List[l_i]),
                                                        l_pCID,
                                                        l_pCDVerHigh,
                                                        l_pCDVerLow);

          if (OK == l_script_status)
          {
            if (l_CD_Continent == l_current_continent)
            {
              GetVersionOnProduct (l_pTargetDrive,
                                   l_pCID,
                                   l_pRT3VerHigh, l_pRT3VerLow);

              l_CID_List[l_i].m_state  = CheckProductVsDVDMediaVersions
                                                           (l_pRT3VerHigh,
                                                            l_pRT3VerLow,
                                                            l_pCDVerHigh,
                                                            l_pCDVerLow);
              if (   (OLDER == l_CID_List[l_i].m_state)
                  || (UP_TO_DATE == l_CID_List[l_i].m_state))
              {
                l_NumberOfCountriesToUpgrade++;
              }
            }
            // Continents are different
            else
            {
              l_CID_List[l_i].m_state = OLDER;
              l_NumberOfCountriesToUpgrade++;
              logMsg ("SOURCE_CONTINENT : l_NumberOfCountriesToUpgrade = %d\n", l_NumberOfCountriesToUpgrade, 0, 0, 0, 0, 0);
            }
            switch (l_CID_List[l_i].m_state)
            {
              case 0:
                UPGLogMsg ("l_CID_List[%d].m_state = NEWER\n", l_i, 0, 0, 0, 0, 0);
                break;
              case 1:
                UPGLogMsg ("l_CID_List[%d].m_state = OLDER\n", l_i, 0, 0, 0, 0, 0);
                break;
              case 2:
                UPGLogMsg ("l_CID_List[%d].m_state = UP_TO_DATE\n", l_i, 0, 0, 0, 0, 0);
                break;
              default:
                UPGLogMsg ("l_CID_List[%d].m_state = UNKNOWN (%d)\n", l_i, l_CID_List[l_i].m_state, 0, 0, 0, 0);
                break;
            }
          }
        }
        UPGLogMsg ("l_NumberOfCountriesToUpgrade         = %d\n", l_NumberOfCountriesToUpgrade, 0, 0, 0, 0, 0);

        if (l_NumberOfCountriesToUpgrade > 0)
        {
          l_bUpgRequired = TRUE;

          if (NULL != l_CID_real_List)
          {
            free (l_CID_real_List);
            l_CID_real_List = NULL;
          }
          l_CID_real_List = (t_CID_List*) malloc (sizeof(t_CID_List) * (l_NumberOfCountriesToUpgrade+1));
          if (NULL == l_CID_real_List)
          {
            UPGLogMsg ("l_CID_real_List == NULL !!!\n", 0, 0, 0, 0, 0, 0);
            l_script_status = ERROR;
          }
          else
          {
            if (l_NumberOfCountriesToUpgrade != DVDMakeCountriesListToUpgrade (&l_DVD_Content,
                                                                  l_NumberOfCountriesToUpgrade,
                                                                  l_CID_real_List))
            {
              UPGLogMsg ("DVDMakeCountriesListToUpgrade ERROR !!!\n", 0, 0, 0, 0, 0, 0);
              l_script_status = ERROR;
            }
          }
        }
        else
        {
          UPGLogMsg ("DB IS UP TO DATE %s.%s -> %s.%s\n", l_pRT3VerHigh, l_pRT3VerLow, l_pCDVerHigh, l_pCDVerLow, 0, 0);
          ClearScreen();
        }
      }
    }

    if (FALSE == FileExists(l_pSourceDrive))
    {
      l_quit_upg = TRUE;
      l_bUpgRequired = FALSE;
    }
    else
    {

      /*** USER CHOICE - BEGIN ***/
      UPGLogMsg ("User choices +, Time = %ld\n", tickGet(), 0, 0, 0, 0, 0);
      // reFill DVD list for display
      if (SOURCE_CONTINENT == l_MediaType)
      {
        UPGLogMsg ("\n=================================\n", 0, 0, 0, 0, 0, 0);
        UPGLogMsg ("SOURCE_CONTINENT == l_MediaType Time = %ld\n", tickGet(), 0, 0, 0, 0, 0);
        UPGLogMsg ("=================================\n", 0, 0, 0, 0, 0, 0);


        strcpy (l_reduced_DVD_Content.m_DVD_title, l_DVD_Content.m_DVD_title);
        l_reduced_DVD_Content.m_DVD_CID_number = l_NumberOfCountriesToUpgrade;
        l_reduced_DVD_Content.m_CID_list       = l_CID_real_List;

        l_quit_upg = TRUE;
        ShowNavigationCountryScreen (&l_reduced_DVD_Content,
                                      l_user_country_name_choice,
                                      l_user_country_CID_choice,
                                     &l_quit_upg);
        UPGLogMsg ("l_user_country_name_choice = '%s'\n", l_user_country_name_choice, 0, 0, 0, 0, 0);
        UPGLogMsg ("l_user_country_CID_choice  = '%s'\n", l_user_country_CID_choice, 0, 0, 0, 0, 0);
        UPGLogMsg ("ShowNavigationCountryScreen done!\n", 0, 0, 0, 0, 0, 0);
        UPGLogMsg ("l_quit_upg = %d\n", l_quit_upg, 0, 0, 0, 0, 0);

        if (FALSE == FileExists(l_pSourceDrive))
        {
          l_quit_upg = TRUE;
        }

        if (TRUE == l_quit_upg)
        {
          l_bUpgRequired = FALSE;
        }
        else
        {
          l_script_status = DVDCheckSpaceAndConfirmUpgrade (l_pSourceDrive,
                                                        l_pTargetDrive,
                                                        l_user_country_CID_choice,
                                                        l_user_country_name_choice,
                                                        l_NumberOfCountriesToUpgrade,
                                                        l_bUpgFormatRequired,
                                                        &l_reduced_DVD_Content,
                                                        &l_bUpgRequired,
                                                        l_MediaInfoType);

          if (  (OK == l_script_status) && (FALSE == l_bUpgRequired) )
          {
            l_quit_upg = FALSE;
          }
        }

        if (OK == l_script_status)
        {
          if (TRUE == l_bUpgRequired)
          {
            ShowNavigationBargraph();
            taskDelay (100);
            ActivateBargraph (l_user_country_name_choice, 0, 0);
          }
        }
        else
        {
          UPGLogMsg ("CheckSpaceAndConfirmUpgrade Error!\n", 0, 0, 0, 0, 0, 0);
        }

      }
      else
      {
        UPGLogMsg ("SOURCE_CONTINENT != l_MediaType Time = %ld\n", tickGet(), 0, 0, 0, 0, 0);

        if (TRUE == l_bUpgRequired)
        {
          if (TRUE == l_bUpgFormatRequired)
          {
            l_quit_upg = FALSE;
          }
          else
          {
            l_quit_upg = TRUE;
          }
            l_script_status = CDCheckSpaceAndConfirmUpgrade (l_pSourceDrive,
                                                             l_path_Data,
                                                            l_pTargetDrive,
                                                            l_pKeyFileName,
                                                            l_pCDVerHigh,
                                                            l_pCDVerLow,
                                                            &l_bUpgRequired,
                                                            l_MediaInfoType,
                                                            l_user_country_name_choice);

        }

        if (OK == l_script_status)
        {
          if (FALSE == l_bUpgRequired)
          {
            UPGLogMsg ("UPGRADE CANCELED BY USER!\n", 0, 0, 0, 0, 0, 0);
            ShowCDCanceled();
            taskDelay (DELAY_5S);
            break;
          }
          else
          {
            UPGLogMsg ("UPGRADE CAN CONTINUE !\n", 0, 0, 0, 0, 0, 0);
            ShowNavigationBargraph();
            taskDelay (100);
            ActivateBargraph (l_user_country_name_choice, 0, 0);
          }
        }
        else
        {
          UPGLogMsg ("CheckSpaceAndConfirmUpgrade Error!\n", 0, 0, 0, 0, 0, 0);
        }

      }
    }

    UPGLogMsg ("User choices -, Time = %ld\n", tickGet(), 0, 0, 0, 0, 0);
    /*** USER CHOICE - END ***/


    //check if upgrade can Continue
    //if (TRUE == CanUpgContinue(l_pSourceDrive, l_quit_upg, l_script_status))
    if (TRUE == CanUpgContinue(l_pSourceDrive, FALSE, l_script_status))
    {
      // Here, we need :
      // - l_MediaType
      // - l_pCID                  in SOURCE_COUNTRY
      // - l_reduced_DVD_Content   in SOURCE_CONTINENT
      // - l_NumberOfCountriesToUpgrade (=1 in SOURCE_COUNTRY)

      //Format partition if continent change
      if (    (OK   == l_script_status)
           && (TRUE == l_bUpgRequired)
           && (TRUE == l_bUpgFormatRequired)
         )
      {
        if (TRUE == l_bUpgFormatRequired)
        {
          strcpy (l_string, "Format SDH!\n");
          UPGLogMsg ("%s", l_string, 0, 0, 0, 0, 0);
          AddUPGHistoryCommentary (l_string);
          UPGStopGuidance ();
          l_script_status = FormatPartNavigation (l_pSourceDrive, l_pTargetDrive);
          if (OK == l_script_status)
          {
            l_MustRebootAfterUpgrade = TRUE;
            l_bUpgFormatRequired = FALSE;
            UPGCheckCarto ();
            if (ERROR == UPGInformNavForNewMapAndWaitAck (l_MediaInfoType))
            {
              l_script_status = ERROR;
            }
          }
        }
      }
    }
    else
    {
      l_script_status = ERROR;
    }


    //check if Drive is Still present
    if (TRUE == CanUpgContinue(l_pSourceDrive, l_quit_upg, l_script_status))
    {
      /*** PREPARE UPGRADE - BEGIN ***/
      if (    (OK    == l_script_status)
           && (TRUE  == l_bUpgRequired))
      {
        UPGLogMsg ("Prepare upgrade +, Time = %ld\n", tickGet(), 0, 0, 0, 0, 0);

        RemoveUnwantedFiles ();

        l_iCountList = 0;
        l_lTotalSize = 0;
        for (l_i=0; l_i<l_NumberOfCountriesToUpgrade; l_i++)
        {
          if (SOURCE_CONTINENT == l_MediaType)
          {
            l_bCIDtoUpdate = DVDGetCIDFromReducedList (&l_reduced_DVD_Content,
                                                        l_i,
                                                        l_pCID);
          }
          else // SOURCE_COUNTRY == l_MediaType
          {
            // l_pCID is OK here
            l_bCIDtoUpdate = TRUE;
          }

          if (TRUE == l_bCIDtoUpdate)
          {
            UPGLogMsg ("- CID '%s' To update\n", (int)l_pCID, 0, 0, 0, 0, 0);
            l_pList = (t_NavUpgData*) realloc (l_pList,
                                                sizeof (t_NavUpgData) * (l_iCountList+1) );
            if (l_pList != NULL)
            {
              strcpy (l_pPath, l_path_Data);
              strcat (l_pPath, "/MAPPE/");
              strcat (l_pPath, l_pCID);
              strcat (l_pPath, KEY_FILE_NAV);
              strcat (l_pPath, ".INF");
              if (ERROR == GetCIDSize (l_pPath, l_pCIDSize, &l_lCIDSize))
              {
                UPGLogMsg ("- GetCIDSize ('%s') return ERROR\n", (int)l_pPath, 0, 0, 0, 0, 0);
              }
              else
              {
                UPGLogMsg ("- GetCIDSize ('%s') return OK\n", (int)l_pPath, 0, 0, 0, 0, 0);
                //add other information

                l_pList[l_iCountList].m_iSize = (int)l_lCIDSize;
                l_lTotalSize += l_pList[l_iCountList].m_iSize;
                strcpy (l_pList[l_iCountList].m_pFileName, "none");
                strcpy (l_pList[l_iCountList].m_pTargetPath, "none");
                strcpy (l_pList[l_iCountList].m_pCID, l_pCID);
              }

              l_iCountList ++;
            }
            else
            {
              UPGLogMsg ("- realloc ERROR\n", 0, 0, 0, 0, 0, 0);
              l_script_status = ERROR;
            }
          }

          UPGLogMsg ("- Total files to be copied = %d\n\n", l_iCountList, 0, 0, 0, 0, 0);
        }

        // Print list
        for (l_i=0; l_i<l_iCountList; l_i++)
        {
          UPGLogMsg ("--> Will be upgraded : %4d SIZE (%9d) CID%s \n",
                  l_i+1,
                  l_pList[l_i].m_iSize,
                  (char*) l_pList[l_i].m_pCID,
                  0, 0, 0);
          taskDelay(10);
        }
        UPGLogMsg ("--> Total size upgraded : %9d bytes\n\n", l_lTotalSize, 0, 0, 0, 0, 0);

        UPGLogMsg ("Prepare upgrade -, Time = %ld\n", tickGet(), 0, 0, 0, 0, 0);
      }

      /*** PREPARE UPGRADE - END ***/

      UPGLogMsg ("==============> Total file size to install %d  (files:%d)\n", l_lTotalSize, l_iCountList, 0, 0, 0, 0);

    }

    // Here, we need :
    // - l_MediaType
    // - l_pSourceDrive
    // - l_pTargetDrive
    // - l_pList                  in SOURCE_CONTINENT
    // - l_iCountList
    // - l_pCID                   in SOURCE_COUNTRY

    /*** UPGRADE - BEGIN ***/
    if (   (TRUE   == CanUpgContinue(l_pSourceDrive, l_quit_upg, l_script_status))
        && (TRUE == l_bUpgRequired))
    {
      UPGLogMsg ("File copy +, Time = %ld\n", tickGet(), 0, 0, 0, 0, 0);

      strcpy (l_pSourcePath, l_pSourceDrive);
      strcat (l_pSourcePath, UPG_DB_DWNL_PPC);
      strcpy (l_pTargetPath, l_pTargetDrive);
      strcat (l_pTargetPath, UPG_DB_DWNL_PPC);
      if (    (TRUE == UPGCheckIfVersionIsNewerThan (l_pSourcePath, l_pTargetPath))
           || (ERROR == CheckCRCFile (l_pTargetPath)))
      {
        if (TRUE == CanUpgContinue (l_pSourceDrive, l_quit_upg, l_script_status))
        {
          l_script_status = CopyFileFromSrcDriveToTargDrive (UPG_DB_DWNL_PPC,
                                                             UPG_DB_DWNL_PPC_TEMP,
                                                             l_pSourceDrive,
                                                             l_pTargetDrive);
        }

        if (TRUE == CanUpgContinue (l_pSourceDrive, l_quit_upg, l_script_status))
        {
          l_script_status = CopyFileFromSrcDriveToTargDrive (UPG_DB_DWNL_PPC_INF,
                                                             UPG_DB_DWNL_PPC_INF_TEMP,
                                                             l_pSourceDrive,
                                                             l_pTargetDrive);
        }
      }

      strcpy (l_pSourcePath, l_pSourceDrive);
      strcat (l_pSourcePath, UPG_GRUPPO_DAT);
      strcpy (l_pTargetPath, l_pTargetDrive);
      strcat (l_pTargetPath, UPG_GRUPPO_DAT);
      if (TRUE == CanUpgContinue (l_pSourceDrive, l_quit_upg, l_script_status))
      {
        if (    (TRUE == UPGCheckIfVersionIsNewerThan (l_pSourcePath, l_pTargetPath))
             || (ERROR == CheckCRCFile (l_pTargetPath)))
        {
          if (TRUE == CanUpgContinue (l_pSourceDrive, l_quit_upg, l_script_status))
          {
            l_script_status = CopyFileFromSrcDriveToTargDrive (UPG_GRUPPO_DAT,
                                                               UPG_GRUPPO_DAT_TEMP,
                                                               l_pSourceDrive,
                                                               l_pTargetDrive);
          }

          if (TRUE == CanUpgContinue (l_pSourceDrive, l_quit_upg, l_script_status))
          {
            l_script_status = CopyFileFromSrcDriveToTargDrive (UPG_GRUPPO_DAT_INF,
                                                               UPG_GRUPPO_DAT_INF_TEMP,
                                                               l_pSourceDrive,
                                                               l_pTargetDrive);
          }

        }

        //Keep key files on HDD
        if (SOURCE_COUNTRY == l_MediaType)
        {
          if (TRUE == CanUpgContinue (l_pSourceDrive, l_quit_upg, l_script_status))
          {
            l_script_status = CopyFileFromSrcDriveToTargDrive (KEY_FILE_NAV,
                                                               KEY_FILE_NAV,
                                                               l_pSourceDrive,
                                                               l_pTargetDrive);
          }

          if (TRUE == CanUpgContinue (l_pSourceDrive, l_quit_upg, l_script_status))
          {
            l_script_status = CopyFileFromSrcDriveToTargDrive (KEY_FILE_LA,
                                                               KEY_FILE_LA,
                                                               l_pSourceDrive,
                                                               l_pTargetDrive);
          }

        }
        else
        {
          if (TRUE == CanUpgContinue (l_pSourceDrive, l_quit_upg, l_script_status))
          {
            l_script_status = CopyFileFromSrcDriveToTargDrive (DVD_KEY_FILE_NAV,
                                                               DVD_KEY_FILE_NAV,
                                                               l_pSourceDrive,
                                                               l_pTargetDrive);
          }

        }
      }


      //Initialize actual CID
      if (l_iCountList != 0)
      {
        if (SOURCE_CONTINENT == l_MediaType)
        {
          strcpy (l_actual_CID, l_pList[0].m_pCID);
        }
        else
        {
          strcpy (l_actual_CID, l_pCID);
        }
        logMsg ("Initial CID = '%s'\n", (char*) l_actual_CID, 0, 0, 0, 0, 0);
      }

      if (OK ==  GetVersionCDOnMedia  (l_pSourceDrive,   l_actual_CID, "CD_VER.LA",
                                              l_pCDVerHighTemp, l_pCDVerLowTemp))
      {
        strcpy (l_string, "Copying of CID ");
        strcat (l_string, l_actual_CID);
        strcat (l_string, " Version : ");
        strcat (l_string, l_pCDVerHighTemp);
        strcat (l_string, ".");
        strcat (l_string, l_pCDVerLowTemp);
        UPGLogMsg ("%s", l_string, 0, 0, 0, 0, 0);
        AddUPGHistoryCommentary (l_string);
      }

      //CID copy
      l_lCompletedSize = 0;
      EnableHistoryHandler (FALSE);
      if (OK == l_script_status)
      {
        ActivateBargraph (l_user_country_name_choice, l_lTotalSize, l_lCompletedSize);
        for (l_i=0; l_i<l_iCountList;  l_i++)
        {
          UPGLogMsg ("File copy begin for CID %s at %ld\n",
                      (char*) l_pList[l_i].m_pCID, tickGet(), 0, 0, 0, 0);
          //uncompress BIN
          strcpy (l_Path_bin_file, l_path_Data);
          strcat (l_Path_bin_file, "/MAPPE/");
          strcat (l_Path_bin_file, l_pList[l_i].m_pCID);
          strcat (l_Path_bin_file, "/");
          strcat (l_Path_bin_file, l_pList[l_i].m_pCID);
          strcat (l_Path_bin_file, ".BIN");

          strcpy (l_dest_folder, l_pTargetDrive);
          strcat (l_dest_folder, "/MAPPE/TEMP_");

          UPGLogMsg ("UnCompress '%s' to '%s'\n",
                      (char*)l_Path_bin_file, (char*)l_dest_folder, 0, 0, 0, 0);

          strcpy (l_Path_gruppo_file, l_path_Data);
          strcat (l_Path_gruppo_file, "/MAPPE/");
          strcat (l_Path_gruppo_file, l_pList[l_i].m_pCID);
          strcat (l_Path_gruppo_file, "/");
          if(OK != UpgIsGruppoValid("GRUPPO_4_CID.DAT.inf",l_Path_gruppo_file,l_pList[l_i].m_pCID))
          {
            l_script_status = ERROR;
          }
          else
          {
            l_script_status = UncompressBinWithTriesIfError (l_Path_bin_file, l_dest_folder);
          }

          //uncompress _EPC.BIN
//AS WE DON NOT YET LOOK FOR DESCRI DAT TO HANDLE THIS, GE (Grand Export) DOESN'T
//HAVE EPC .BIN FILE. SO WE NEED TO CHECK FIRST THAT THE FILE IS PRESENT.
//IN SOME OTHER CASE THE PRESENCE OF THIS FILE MEANS ALSO THAT IT IS PRESENT IN DESCRI.DAT
          if (OK == l_script_status)
          {
            strcpy (l_Path_bin_file, l_path_Data);
            strcat (l_Path_bin_file, "/MAPPE/");
            strcat (l_Path_bin_file, l_pList[l_i].m_pCID);
            strcat (l_Path_bin_file, "/");
            strcat (l_Path_bin_file, l_pList[l_i].m_pCID);
            strcat (l_Path_bin_file, "_EPC.BIN");
            //check if file exist
            if (TRUE == FileExists(l_Path_bin_file))
            {
              l_script_status = UncompressBinWithTriesIfError (l_Path_bin_file,
                                                               l_dest_folder);

              //copy {CID}_EPC.BIN.inf file
              if (OK == l_script_status)
              {
                strcat (l_Path_bin_file, ".inf");

                //
                strcpy (l_Path_dest_bin_file, l_dest_folder);
                strcat (l_Path_dest_bin_file, l_pList[l_i].m_pCID);
                strcat (l_Path_dest_bin_file, "/");
                strcat (l_Path_dest_bin_file, l_pList[l_i].m_pCID);
                strcat (l_Path_dest_bin_file, "_EPC.BIN.inf");
                UPGLogMsg (" Start Copy ('%s' to '%s') \n",
                           (int) l_Path_bin_file, (int) l_Path_dest_bin_file, 0, 0, 0, 0);

                do
                {
                  l_script_status = UPGCopy (l_Path_bin_file, l_Path_dest_bin_file, FALSE);
                  if (ERROR == l_script_status)   // UPGCopy  failed
                  {
                    UPGLogMsg ("[ERROR] - Copy '%s' to '%s' : FAILED!  Trial:%d\n",
                               (int) l_Path_bin_file,
                               (int) l_Path_dest_bin_file,
                               ((UPG_MAX_TRIALS-l_nb_trials)+1), 0, 0, 0);
                  }
                  else
                  {
                    UPGLogMsg ("Copy '%s' to '%s' : OK\n",
                               (int) l_Path_bin_file,
                               (int) l_Path_dest_bin_file, 0, 0, 0, 0);
                    l_script_status = OK;
                    break;
            }
                  l_nb_trials--;

          }
                while (l_nb_trials > 0);

              }
            }
          }

          if (ERROR == l_script_status)
          {
            UPGLogMsg ("UnCompress Error: %s to %s\n",
                        (char*)l_Path_bin_file, (char*)l_dest_folder, 0, 0, 0, 0);
            EnableHistoryHandler (TRUE);
            strcpy (l_string, "- Copy error on '");
            strcat (l_string, l_Path_bin_file);
            strcat (l_string, "'");
            AddUPGHistoryCommentary (l_string);
            break;
          }
          else
          {
            strcpy (l_Path_bin_file, l_path_Data);
            strcat (l_Path_bin_file, "/MAPPE/");
            strcat (l_Path_bin_file, l_pList[l_i].m_pCID);
            strcat (l_Path_bin_file, "/");

            strcpy (l_dest_folder, l_pTargetDrive);
            strcat (l_dest_folder, "/MAPPE/TEMP_");
            strcat (l_dest_folder, l_pList[l_i].m_pCID);
            strcat (l_dest_folder, "/");

            l_script_status = CopyFileFromSrcDriveToTargDrive ("CD_VER.LA.INF", "CD_VER.LA.INF",
                                        l_Path_bin_file, l_dest_folder);
            if (OK == l_script_status)
            {
              l_script_status = CopyFileFromSrcDriveToTargDrive ("CD_VER.LA.INF.inf", "CD_VER.LA.INF.inf",
                                        l_Path_bin_file, l_dest_folder);
            }
            if (OK == l_script_status)
            {
              l_script_status = CopyFileFromSrcDriveToTargDrive ("CD_VER.NAV.INF", "CD_VER.NAV.INF",
                                        l_Path_bin_file, l_dest_folder);
            }
            if (OK == l_script_status)
            {
              l_script_status = CopyFileFromSrcDriveToTargDrive ("CD_VER.NAV.INF.inf", "CD_VER.NAV.INF.inf",
                                        l_Path_bin_file, l_dest_folder);
            }
            if (OK == l_script_status)
            {
              l_script_status = CopyFileFromSrcDriveToTargDrive ("GRUPPO_4_CID.DAT", "GRUPPO_4_CID.DAT",
                                        l_Path_bin_file, l_dest_folder);
            }
            if (OK == l_script_status)
            {
              l_script_status = CopyFileFromSrcDriveToTargDrive ("GRUPPO_4_CID.DAT.inf", "GRUPPO_4_CID.DAT.inf",
                                        l_Path_bin_file, l_dest_folder);
            }
            else
            {
              l_script_status = ERROR;
              break;
            }
          }

          UPGLogMsg ("File copy end for CID %s at %ld\n",
                        (char*) l_pList[l_i].m_pCID, tickGet(), 0, 0, 0, 0);
          EnableHistoryHandler (TRUE);

          if (1 == l_iCountList)
          {
            StepBargraph (l_user_country_name_choice, 100);
          }

          if (ERROR == UPGInformNavForNewMapAndWaitAck (l_MediaInfoType))
          {
            strcpy (l_string, "- UPGInformNavForNewMapAndWaitAck ERROR for CID '");
            strcat (l_string, l_pList[l_i].m_pCID);
            strcat (l_string, "'");
            AddUPGHistoryCommentary (l_string);
            l_script_status = ERROR;
            break;
          }
          else
          {
            strcpy (l_string, "- UPGInformNavForNewMapAndWaitAck OK for CID '");
            strcat (l_string, l_pList[l_i].m_pCID);
            strcat (l_string, "'");
            AddUPGHistoryCommentary (l_string);
          }
          EnableHistoryHandler (FALSE);

          l_lCompletedSize += l_pList[l_i].m_iSize;
          ActivateBargraph (l_user_country_name_choice, l_lTotalSize, l_lCompletedSize);

          CleanTemporaryFiles (l_pTargetDrive, l_pList[l_i].m_pCID);
        }
      }
      EnableHistoryHandler (TRUE);

      UPGLogMsg ("File copy -, Time = %ld\n", tickGet(), 0, 0, 0, 0, 0);
    }
    /*** UPGRADE - END ***/

    DeactivateBargraph ();

    /*** COPY OK ***/
    if (SOURCE_CONTINENT == l_MediaType)
    {
      if ( (OK == l_script_status) && (TRUE == l_bUpgRequired))
      {
        for (l_i=0 ;l_i<l_NumberOfCountriesToUpgrade; l_i++)
        {
          if (TO_UPDATE == l_reduced_DVD_Content.m_CID_list[l_i].m_state)
          {
            l_CID_real_List[l_i].m_state = UP_TO_DATE ;
          }

          switch (l_reduced_DVD_Content.m_CID_list[l_i].m_state)
          {
            case 0: // NEWER
              UPGLogMsg ("l_CID_real_List[%d].m_state = NEWER\n", l_i, 0, 0, 0, 0, 0);
              break;
            case 1: // OLDER
              UPGLogMsg ("l_CID_real_List[%d].m_state = OLDER\n", l_i, 0, 0, 0, 0, 0);
              break;
            case 2: // UP_TO_DATE
              UPGLogMsg ("l_CID_real_List[%d].m_state = UP_TO_DATE\n", l_i, 0, 0, 0, 0, 0);
              break;
            default:
              UPGLogMsg ("l_CID_real_List[%d].m_state = UNKNOWN (%d) !!!\n", l_i, l_reduced_DVD_Content.m_CID_list[l_i].m_state, 0, 0, 0, 0);
              break;
          }
        }
      }
    }
    else
    {
      // SOURCE_COUNTRY== l_MediaType : if country copied, then quit upgrade.
      l_quit_upg = TRUE;
    }
  }


  if (NULL != l_CID_List)
  {
    //   l_DVD_Content.m_CID_list = l_CID_List ;
    free (l_CID_List);
    l_CID_List = NULL;
  }

  if (NULL != l_CID_real_List)
  {
    free (l_CID_real_List);
    l_CID_real_List = NULL;
  }

  if (NULL != l_pList)
  {
    free (l_pList);
    l_pList = NULL;
  }

  //remove of temp file  if failure
  UPGLogMsg ("Delete all \"/SDH/TEMP_*\" files\n", 0, 0, 0, 0, 0, 0);
  xdelete ("/SDH/TEMP_*");


  /*** TERMINATION ***/

  if (OK == l_script_status)
  {
    AddUPGHistoryCommentary ("-> UPGRADE OK !");
    UPGLogMsg ("UPGRADE ENDED WITHOUT ERROR!\n", 0, 0, 0, 0, 0, 0);
  }
  else
  {
    if (FALSE == FileExists(l_pSourceDrive))
    {
      AddUPGHistoryCommentary ("-> UPGRADE KEY HAS BEEN REMOVED !");
      UPGLogMsg ("UPGRADE ENDED WITHOUT ERROR!\n", 0, 0, 0, 0, 0, 0);
    }
    else if (TRUE == l_quit_upg)
    {
      AddUPGHistoryCommentary ("-> UPGRADE CANCELLED BY USER !");
      UPGLogMsg ("UPGRADE ENDED WITHOUT ERROR!\n", 0, 0, 0, 0, 0, 0);
    }
    else
    {
      AddUPGHistoryCommentary ("-> UPGRADE ERROR !");
      UPGLogMsg ("SCRIPT ERROR!\n", 0, 0, 0, 0, 0, 0);
    }
  }

  if(TRUE == l_MustRebootAfterUpgrade)
  {
    RebootSystem();
  }

  // Stop logging
  EnableHistoryHandler (FALSE);
  SetUpgradeLogging (FALSE);

  if (ERROR == l_script_status)
  {
    ShowCDProblem ();
    taskDelay (DELAY_5S);
  }

  ClearScreen ();
  taskDelay (100);


  return (l_script_status);
}


/********************** INTERN FUNCTION DEFINITION **********************/

STATUS  ComputeNavTargetPath (char *p_pTargetPath,
                              char *p_pSourcePath,
                              char *p_pCID,
                              char *p_pSourceDrive,
                              char *p_pTargetDrive)
{
  STATUS l_status = OK;
  char l_pFileName[MAX_PATH_LENGTH];
  char l_pTargetPath[MAX_PATH_LENGTH];
  char l_pCID[MAX_FILENAME_LENGTH];
  char l_pTEMP_CID[MAX_FILENAME_LENGTH];

  if (TRUE == IsOnRoot (p_pSourcePath))
  {
    if (ERROR == GetFileName (p_pSourcePath, l_pFileName))
    {
      UPGLogMsg ("GetFileName error on %s\n", (char*) p_pSourcePath, 0, 0, 0, 0, 0);
      l_status = ERROR;
    }
    else
    {
      strcpy (p_pTargetPath, p_pTargetDrive);
      strcat (p_pTargetPath, "/TEMP_");
      strcat (p_pTargetPath, l_pFileName);
    }
  }
  else
  {
    strcpy (l_pTargetPath, p_pSourcePath);

    //Replace drive
    if (ERROR == ReplaceInPath (l_pTargetPath, p_pSourceDrive, p_pTargetDrive))
    {
      l_status = ERROR;
    }
    else
    {
      //Replace CID -> TEMP_CID
      strcpy (l_pCID, "/");
      strcat (l_pCID, p_pCID);
      strcat (l_pCID, "/");

      strcpy (l_pTEMP_CID, "/TEMP_");
      strcat (l_pTEMP_CID, p_pCID);
      strcat (l_pTEMP_CID, "/");

      if (ERROR == ReplaceInPath (l_pTargetPath, l_pCID, l_pTEMP_CID))
      {
        l_status = ERROR;
      }
    }
    strcpy (p_pTargetPath, l_pTargetPath);
  }

  return (l_status);
}


int IsOnRoot (char *p_pPath)
{
  int l_iCountSlash = 0;
  int l_iLength = strlen (p_pPath);
  int l_i = 0;

  for (l_i=0; l_i<l_iLength; l_i++)
  {
    if ('/' == p_pPath[l_i])
    {
      l_iCountSlash++;
    }
  }

  if (2 == l_iCountSlash)
  {
    return (TRUE);
  }
  else
  {
    return (FALSE);
  }
}


STATUS ReplaceInPath (char *p_pPath, char *p_pStringToReplace, char *p_pStringToReplaceWhith)
{
  int l_iLenPath = 0;
  int l_iLenStrToReplace = 0;
  int l_iLenStrToReplaceWith = 0;

  char *l_pPrefPtr = NULL;
  char *l_pPostPtr = NULL;
  char l_pPath[MAX_PATH_LENGTH];

  char l_pPostString[MAX_PATH_LENGTH];


  //Check parameters
  if ( (NULL == p_pPath) || (NULL == p_pStringToReplace) || (NULL == p_pStringToReplaceWhith))
  {
    UPGLogMsg ("ReplaceInPath: NULL param\n", 0, 0, 0, 0, 0, 0);
    return (ERROR);
  }

  l_iLenPath = strlen (p_pPath);
  l_iLenStrToReplace = strlen (p_pStringToReplace);
  l_iLenStrToReplaceWith = strlen (p_pStringToReplaceWhith);

  //Check substring lengths
  if ( (l_iLenStrToReplace > l_iLenPath) || (l_iLenStrToReplaceWith > l_iLenPath))
  {
    UPGLogMsg ("ReplaceInPath: substring is longer than string\n", 0, 0, 0, 0, 0, 0);
    return (ERROR);
  }

  //Local copy
  strcpy (l_pPath, p_pPath);

  //Look for pref ptr
  l_pPrefPtr = strstr (l_pPath, p_pStringToReplace);
  if (NULL == l_pPrefPtr)
  {
    UPGLogMsg ("ReplaceInPath: substring not founded\n", 0, 0, 0, 0, 0, 0);
    return (ERROR);
  }

  //Look for post ptr
  l_pPostPtr = l_pPrefPtr + l_iLenStrToReplace;

  //Get post string
  strcpy (l_pPostString, l_pPostPtr);

  //Replace substring
  strcpy (l_pPrefPtr, p_pStringToReplaceWhith);

  //Append post string
  strcat (l_pPath, l_pPostString);

  //Copy back
  strcpy (p_pPath, l_pPath);

  return (OK);
}


int FileExists (char *p_pPath)
{
  FILE* l_pFich = NULL;

  if (NULL == (l_pFich = fopen (p_pPath, "r")))
  {
    return (FALSE);
  }
  else
  {
    fclose (l_pFich);
    return (TRUE);
  }
}


STATUS GetFileName (char *p_pPath, char *p_pFileName)
{
  STATUS l_status = OK;

  int l_i = strlen (p_pPath);
  while (l_i > 0)
  {
    if ('/' == p_pPath[l_i])
    {
      strcpy (p_pFileName, p_pPath + l_i + 1);
      break;
    }
    else
    {
      l_i--;
    }
  }

  return (l_status);
}


void CleanTemporaryFiles (char *p_pTargetDrive, char *p_pCID)
{
  char l_pTargetPath[MAX_PATH_LENGTH];

  strcpy (l_pTargetPath, p_pTargetDrive);
  strcat (l_pTargetPath, "/TEMP_NAV_UPGRADE.CMD");
  UPGDelete (l_pTargetPath);

  strcpy (l_pTargetPath, p_pTargetDrive);
  strcat (l_pTargetPath, "/TEMP_CD_VER.NAV");
  UPGDelete (l_pTargetPath);

  strcpy (l_pTargetPath, p_pTargetDrive);
  strcat (l_pTargetPath, "/TEMP_CD_VER.LA");
  UPGDelete (l_pTargetPath);

  strcpy (l_pTargetPath, p_pTargetDrive);
  strcat (l_pTargetPath, "/TEMP_DVD_VER.NAV");
  UPGDelete (l_pTargetPath);

  strcpy (l_pTargetPath, p_pTargetDrive);
  strcat (l_pTargetPath, UPG_DB_DWNL_PPC_TEMP);
  UPGDelete (l_pTargetPath);
  strcat (l_pTargetPath, ".inf");
  UPGDelete (l_pTargetPath);

  strcpy (l_pTargetPath, p_pTargetDrive);
  strcat (l_pTargetPath, "/TEMP_GRUPPO_4_ROOT.DAT");
  UPGDelete (l_pTargetPath);
  strcat (l_pTargetPath, ".inf");
  UPGDelete (l_pTargetPath);

  strcpy (l_pTargetPath, p_pTargetDrive);
  strcat (l_pTargetPath, "/TEMP_GRUPPO_4_ROOT.DAT.inf");
  UPGDelete (l_pTargetPath);
  strcat (l_pTargetPath, ".inf");
  UPGDelete (l_pTargetPath);

  strcpy (l_pTargetPath, p_pTargetDrive);
  strcat (l_pTargetPath, "/MAPPE/TEMP_");
  strcat (l_pTargetPath, p_pCID);
  xdelete (l_pTargetPath);
  UPGMKDelete (l_pTargetPath);

  strcpy (l_pTargetPath, p_pTargetDrive);
  strcat (l_pTargetPath, "/TEMP_SW_VER.DAT");
  UPGDelete (l_pTargetPath);
}


t_continent GetContinent (int p_cid)
{
  t_continent l_continent = UNKNOWN_CONTINENT;

  switch (p_cid)
  {
  case 1 :
    l_continent = EUROPE;
    break;
  case 2 :
    l_continent = EUROPE;
    break;
  case 3 :
    l_continent = EUROPE;
    break;
  case 4 :
    l_continent = EUROPE;
    break;
  case 5 :
    l_continent = EUROPE;
    break;
  case 6 :
    l_continent = EUROPE;
    break;
  case 11 :
    l_continent = EUROPE;
    break;
  case 12 :
    l_continent = EUROPE;
    break;
  case 13 :
    l_continent = EUROPE;
    break;
  case 16 :
    l_continent = EUROPE;
    break;
  case 20 :
    l_continent = EUROPE;
    break;
  case 21 :
    l_continent = EUROPE;
    break;
  case 22 :
    l_continent = EUROPE;
    break;
  case 24 :
    l_continent = EUROPE;
    break;

  case 23 :
    l_continent = FAR_EAST;
    break;

  case 25 :
    l_continent = AUSTRALIA;
    break;
  case 26 :
    l_continent = AUSTRALIA;
    break;

  case 27 :
    l_continent = SOUTH_AFRICA;
    break;

  case 7 :
    l_continent = SOUTH_AMERICA;
    break;
  case 29 :
    l_continent = SOUTH_AMERICA;
    break;

  case 8 :
    l_continent = NORTH_AMERICA;
    break;
  case 9 :
    l_continent = NORTH_AMERICA;
    break;
  case 10 :
    l_continent = NORTH_AMERICA;
    break;
  case 14 :
    l_continent = NORTH_AMERICA;
    break;
  case 15 :
    l_continent = NORTH_AMERICA;
    break;
  case 18 :
    l_continent = NORTH_AMERICA;
    break;
  case 19 :
    l_continent = NORTH_AMERICA;
    break;
  case 28 :
    l_continent = NORTH_AMERICA;
    break;

  case 17 :
    l_continent = MIDDLE_EAST;
    break;

  case 30 :
    l_continent = CHINA;
    break;

  case 31 :
    l_continent = NEW_ZEALAND;
    break;

  case 32:
    l_continent = INDIA;
    break;

  case 33:
    l_continent = COMPLETE_RUSSIA;
    break;

  case 34:
    l_continent = NORTH_AFRICA;
    break;

  case 35:
    l_continent = NORTH_AFRICA;
    break;

  case 36:
    l_continent = CENTRAL_AMERICA;
    break;

  case 39:
    l_continent = JAPAN;
    break;

  default :
    l_continent = UNKNOWN_CONTINENT;
    break;
  }

  return (l_continent);
}


t_continent GetInstalledContinent (void)
{
  DIR* l_dir = NULL;
  t_continent l_continent = UNKNOWN_CONTINENT;

  if (NULL !=      (l_dir = opendir ("/SDH/Mappe/001")))
  {
    l_continent = EUROPE;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/002")))
  {
    l_continent = EUROPE;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/003")))
  {
    l_continent = EUROPE;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/004")))
  {
    l_continent = EUROPE;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/005")))
  {
    l_continent = EUROPE;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/006")))
  {
    l_continent = EUROPE;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/007")))
  {
    l_continent = SOUTH_AMERICA;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/008")))
  {
    l_continent = NORTH_AMERICA;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/009")))
  {
    l_continent = NORTH_AMERICA;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/010")))
  {
    l_continent = NORTH_AMERICA;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/011")))
  {
    l_continent = EUROPE;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/012")))
  {
    l_continent = EUROPE;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/013")))
  {
    l_continent = EUROPE;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/014")))
  {
    l_continent = NORTH_AMERICA;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/015")))
  {
    l_continent = NORTH_AMERICA;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/016")))
  {
    l_continent = EUROPE;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/017")))
  {
    l_continent = MIDDLE_EAST;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/018")))
  {
    l_continent = NORTH_AMERICA;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/019")))
  {
    l_continent = NORTH_AMERICA;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/020")))
  {
    l_continent = EUROPE;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/021")))
  {
    l_continent = EUROPE;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/022")))
  {
    l_continent = EUROPE;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/023")))
  {
    l_continent = FAR_EAST;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/024")))
  {
    l_continent = EUROPE;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/025")))
  {
    l_continent = AUSTRALIA;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/026")))
  {
    l_continent = AUSTRALIA;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/027")))
  {
    l_continent = SOUTH_AFRICA;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/028")))
  {
    l_continent = NORTH_AMERICA;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/029")))
  {
    l_continent = SOUTH_AMERICA;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/030")))
  {
    l_continent = CHINA;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/031")))
  {
    l_continent = NEW_ZEALAND;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/032")))
  {
    l_continent = INDIA;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/033")))
  {
    l_continent = COMPLETE_RUSSIA;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/034")))
  {
    l_continent = NORTH_AFRICA;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/035")))
  {
    l_continent = NORTH_AFRICA;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/036")))
  {
    l_continent = CENTRAL_AMERICA;
  }
  else if (NULL != (l_dir = opendir ("/SDH/Mappe/039")))
  {
    l_continent = JAPAN;
  }

  if (NULL != l_dir)
  {
    closedir (l_dir);
  }

  return (l_continent);
}


STATUS ExtractFileInfos ( char          *p_pSourceDrive,
                          char          *p_filename,
                          t_DVD_Content *p_dvd_content)
{
  STATUS l_status = OK;
  FILE  *l_file = NULL;

  char   l_line[MAX_INF_LINE_LENGTH];
  char  *l_ptr = NULL;
  char  *l_ptr1 = NULL;
  char   l_value[MAX_VERSION_LENGTH];
  char   l_CIDValue[MAX_CID_LENGTH];
  char   l_CID_EPC_Value[MAX_CID_LENGTH];
  char   l_CDVersion[MAX_VERSION_LENGTH];
  char   l_CDRelease[MAX_VERSION_LENGTH];
  char   l_QVersion[MAX_VERSION_LENGTH];
  char   l_year[MAX_VERSION_LENGTH];
  char   l_date[MAX_VERSION_LENGTH];
  char   l_country_name[MAX_VERSION_LENGTH];
  char   l_pPath[MAX_PATH_LENGTH];
  char   l_pCIDSize[MAX_CID_SIZE_LENGTH];
  unsigned long l_lCID_Size = 0;
  unsigned long l_lCID_EPC_Size = 0;
  int    l_counter = 0 ;
  int    l_value_size = 0 ;
  t_CID_List *l_CID_List = NULL ;

  if (NULL == p_filename || NULL == p_dvd_content)
  {
    UPGLogMsg ("ExtractFileInfos: filename or p_dvd_content is NULL\n", 0, 0, 0, 0, 0, 0);
    l_status = ERROR ;
  }
  else
  {
    UPGLogMsg ("ExtractFileInfos: Try to open ('%s')\n",
               (int) p_filename, 0, 0, 0, 0, 0);
    l_file = fopen ( p_filename, "r");

    if (NULL == l_file)
    {
      l_status = ERROR ;
      UPGLogMsg ("ExtractFileInfos: open '%s' ERROR\n", (int) p_filename, 0, 0, 0, 0, 0);
    }
    else
    {
      while (NULL != fgets (l_line, MAX_INF_LINE_LENGTH, l_file))
      {
        // Only if 'CID' is at the beginning of the line
        if (l_line == strstr (l_line, "CID"))
        {
          l_counter++;
        }
      }

      if (l_counter > 0)
      {
        //allocate DVD table
        UPGLogMsg ("ExtractFileInfos: number of CID = %d\n", l_counter, 0, 0, 0, 0, 0);
        p_dvd_content->m_DVD_CID_number = l_counter;

        l_CID_List = (t_CID_List*) malloc (sizeof (t_CID_List) * (l_counter+1));
        logMsg ("ExtractFileInfos: _sizeof(t_CID_List) = %d\n", sizeof (t_CID_List), 0, 0, 0, 0, 0);
        p_dvd_content->m_CID_list = l_CID_List;

      }
      else
      {
        l_status = ERROR ;
      }

      if ( (ERROR == l_status) || (NULL == l_CID_List))
      {
        UPGLogMsg ("ExtractFileInfos: allocate p_dvd_content->l_CID_List ERROR\n", 0, 0, 0, 0, 0, 0);
        l_status = ERROR ;
      }
      else
      {
        fseek (l_file, 0, SEEK_SET);

        l_counter = 0 ;
        //Look for field in file
        while ( (NULL != fgets (l_line, MAX_INF_LINE_LENGTH, l_file)) && (OK == l_status))
        {
          // Verify CID existance
          l_ptr = strstr (l_line, "CID");
          // Only if 'CID' is at the beginning of the line
          if (l_line == l_ptr)
          {
            l_counter++;
            UPGLogMsg ("ExtractFileInfos:  l_counter    : %d\n", l_counter, 0, 0, 0, 0, 0);

            // Extract CID
            l_ptr = strstr (l_line, ":") + 1;
            l_ptr1 = strstr (l_ptr, ",");
            l_value_size = l_ptr1 - l_ptr ;
            if (l_value_size <= 0)
            {
              UPGLogMsg (" ExtractFileInfos: CID not found\n", 0, 0, 0, 0, 0, 0);
              l_status = ERROR ;
            }
            else
            {
              strncpy (l_CIDValue, l_ptr, l_value_size);
              l_CIDValue[l_value_size] = '\0';
            }

            // Extract CID version
            if (OK == l_status)
            {
              l_ptr = l_ptr1 + 1 ;
              l_ptr1 = strstr (l_ptr, ",");
              l_value_size = l_ptr1 - l_ptr ;
              if (l_value_size <= 0)
              {
                UPGLogMsg (" ExtractFileInfos: CID version  : CID Version not found \n", 0, 0, 0, 0, 0, 0);
                l_status = ERROR ;
              }
              else
              {
                strncpy (l_CDVersion, l_ptr, l_value_size);
                l_CDVersion[l_value_size] = '\0';
                UPGLogMsg (" ExtractFileInfos: CID version  : '%s'\n", (int) l_CDVersion, 0, 0, 0, 0, 0);
              }
            }

            // Extract CID release
            if (OK == l_status)
            {
              l_ptr = l_ptr1 + 1 ;
              l_ptr1 = strstr (l_ptr, ",");
              l_value_size = l_ptr1 - l_ptr ;
              if (l_value_size <= 0)
              {
                UPGLogMsg (" ExtractFileInfos: CID release  : CID Release not found \n", 0, 0, 0, 0, 0, 0);
                l_status = ERROR ;
              }
              else
              {
                strncpy (l_CDRelease, l_ptr, l_value_size);
                l_CDRelease[l_value_size] = '\0';
                UPGLogMsg (" ExtractFileInfos: CID release  : '%s'\n", (int) l_CDRelease, 0, 0, 0, 0, 0);
              }
            }

            // Extract Q version
            if (OK == l_status)
            {
              l_ptr = l_ptr1 + 1 ;
              l_ptr1 = strstr (l_ptr, ",");
              l_value_size = l_ptr1 - l_ptr ;
              if (l_value_size <= 0)
              {
                UPGLogMsg (" ExtractFileInfos: Q version    : Q not found \n", 0, 0, 0, 0, 0, 0);
                l_status = ERROR ;
              }
              else
              {
                strncpy (l_QVersion, l_ptr, l_value_size);
                l_QVersion[l_value_size] = '\0';
                UPGLogMsg (" ExtractFileInfos: Q version    : '%s'\n", (int) l_QVersion, 0, 0, 0, 0, 0);
              }
            }

            // Extract Year
            if (OK == l_status)
            {
              l_ptr = l_ptr1 + 1 ;
              l_ptr1 = strstr (l_ptr, ",");
              l_value_size = l_ptr1 - l_ptr ;
              if (l_value_size <= 0)
              {
                UPGLogMsg (" ExtractFileInfos: Year         : Year not found \n", 0, 0, 0, 0, 0, 0);
                l_status = ERROR ;
              }
              else
              {
                strncpy (l_year, l_ptr, l_value_size);
                l_year[l_value_size] = '\0';
                UPGLogMsg (" ExtractFileInfos: Year         : '%s'\n", (int) l_year, 0, 0, 0, 0, 0);
              }
            }

            // Extract date
            if (OK == l_status)
            {
              l_ptr = strstr (l_ptr1 + 1, "\"") + 1;
              l_ptr1 = strstr (l_ptr, "\"");
              l_value_size = l_ptr1 - l_ptr ;
              if (l_value_size <= 0)
              {
                UPGLogMsg (" ExtractFileInfos: Date         : Date not found \n", 0, 0, 0, 0, 0, 0);
                l_status = ERROR ;
              }
              else
              {
                strncpy (l_date, l_ptr, l_value_size);
                l_date[l_value_size] = '\0';
                UPGLogMsg (" ExtractFileInfos: Date         : '%s'\n", (int) l_date, 0, 0, 0, 0, 0);
              }
            }

            // Extract Country Name
            if (OK == l_status)
            {
              l_ptr = strstr (l_ptr1 + 1, "\"") + 1;
              l_ptr1 = strstr (l_ptr, "\"");
              l_value_size = l_ptr1 - l_ptr ;
              if (l_value_size <= 0)
              {
                UPGLogMsg (" ExtractFileInfos: Country name : Country name not found \n", 0, 0, 0, 0, 0, 0);
                l_status = ERROR ;
              }
              else
              {
                strncpy (l_country_name, l_ptr, l_value_size);
                l_country_name[l_value_size] = '\0';
                UPGLogMsg (" ExtractFileInfos: Country name : '%s'\n", (int) l_country_name, 0, 0, 0, 0, 0);
              }
            }

            // Extract cid_size from the "CID"/CD_VER.NAV.INF file...
            // Get the DB version on RT3
            strcpy (l_pPath, p_pSourceDrive);
            strcat (l_pPath, "/MAPPE/");
            strcat (l_pPath, l_CIDValue);
            strcat (l_pPath, KEY_FILE_NAV);
            strcat (l_pPath, ".INF");
            //
            if (ERROR == GetCIDSize (l_pPath, l_pCIDSize, &l_lCID_Size))
            {
              UPGLogMsg (" GetCIDSize: cid size KO     : '%s'\n", (int) l_pPath, 0, 0, 0, 0, 0);
              strcpy (l_pCIDSize, "0");
              l_lCID_Size = 0;
            }
            else
            {
              UPGLogMsg (" ExtractFileInfos: l_pCIDSize: '%s', l_lCID_Size=%d \n",
                        (int)l_pCIDSize, (int)l_lCID_Size, 0, 0, 0, 0);
            }


            // Extract cid_size from the "CID"/{CID}_EPC.BIN.inf file...
            // Get the DB version on RT3
            strcpy (l_pPath, p_pSourceDrive);
            strcat (l_pPath, "/MAPPE/");
            strcat (l_pPath, l_CIDValue);
            strcat (l_pPath, "/");
            strcat (l_pPath, l_CIDValue);
            strcat (l_pPath, "_EPC.BIN.inf");

            l_lCID_EPC_Size = 0;

            //check first if the file exist
            if (TRUE == FileExists(l_pPath))
            {
              if (ERROR == GetCIDSize (l_pPath, l_CID_EPC_Value, &l_lCID_EPC_Size))
              {
                UPGLogMsg (" GetCIDSize: EPC cid size KO     : '%s'\n", (int) l_pPath, 0, 0, 0, 0, 0);
                strcpy (l_CID_EPC_Value, "0");
                l_lCID_EPC_Size = 0;
              }
              else
              {
                UPGLogMsg (" ExtractFileInfos: l_pCIDSize for EPC: '%s', l_lCID_Size=%d \n",
                          (int)l_CID_EPC_Value, (int)l_lCID_EPC_Size, 0, 0, 0, 0);
              }
            }

            // Fill DVD list content
            if (OK == l_status)
            {
              strcpy (p_dvd_content->m_CID_list[l_counter-1].m_pCID,        l_CIDValue) ;
              strcpy (p_dvd_content->m_CID_list[l_counter-1].m_pCDVersion,  l_CDVersion) ;
              strcpy (p_dvd_content->m_CID_list[l_counter-1].m_pCDRelease,  l_CDRelease) ;
              strcpy (p_dvd_content->m_CID_list[l_counter-1].m_pHDDVersion, "0") ;
              strcpy (p_dvd_content->m_CID_list[l_counter-1].m_pHDDRelease, "0") ;
              p_dvd_content->m_CID_list[l_counter-1].m_state =              NEWER;
              strcpy (p_dvd_content->m_CID_list[l_counter-1].m_CID_name,    l_country_name) ;
              p_dvd_content->m_CID_list[l_counter-1].m_cid_size = (l_lCID_Size + l_lCID_EPC_Size);
              UPGLogMsg (" fill DVD list Content: CID='%s', l_lCID_n_EPC_Size=%d \n",
                        (int)l_CIDValue,
                        (int)(p_dvd_content->m_CID_list[l_counter-1].m_cid_size),
                        0, 0, 0, 0);
            }
          }
        }
      }
      fclose (l_file);
    }
  }

  return (l_status);
}



STATUS CheckSourceIntegrity (char *p_pSourceDrive)
{
  STATUS l_status = OK;
  char l_pSourcePath[MAX_PATH_LENGTH];

  UPGLogMsg ("CheckSourceIntegrity +, Time = %ld\n", tickGet(), 0, 0, 0, 0, 0);

  strcpy (l_pSourcePath, p_pSourceDrive);
  strcat (l_pSourcePath, UPG_DB_DWNL_PPC);

  if (ERROR == CheckCRCFile (l_pSourcePath))
  {
    UPGLogMsg ("CheckCRCFile failed on <%s>\n", l_pSourcePath, 0, 0, 0, 0, 0);
    l_status = ERROR;
  }
  else
  {
    UPGLogMsg ("CheckCRCFile OK on <%s>\n", l_pSourcePath, 0, 0, 0, 0, 0);
  }

  if (OK == l_status)
  {
    strcpy (l_pSourcePath, p_pSourceDrive);
    strcat (l_pSourcePath, UPG_GRUPPO_DAT);

    if (ERROR == CheckCRCFile (l_pSourcePath))
    {
      UPGLogMsg ("CheckCRCFile failed on <%s>\n", l_pSourcePath, 0, 0, 0, 0, 0);
      l_status = ERROR;
    }
    else
    {
      UPGLogMsg ("CheckCRCFile OK on <%s>\n", l_pSourcePath, 0, 0, 0, 0, 0);
    }
  }

  UPGLogMsg ("CheckSourceIntegrity -, Time = %ld\n", tickGet(), 0, 0, 0, 0, 0);

  return (l_status);
}



STATUS GetInfosOnMedia (t_Source       l_MediaType,
                        char*          p_path_Data,
                        t_DVD_Content *l_DVD_Content,
                        int           *l_CID_Count,
                        t_CID_List    **l_CID_List)
{
  STATUS  l_script_status = ERROR;
  char    l_filename[MAX_PATH_LENGTH];
  char l_currVersNaviFile[CMD_MEDIA_TYPE_MAX_SIZE];


  //Get number of CIDs if DVD
  if (SOURCE_COUNTRY== l_MediaType)
  {
    *l_CID_Count = 1;
    l_script_status = OK;
    l_script_status = GetUpgDVD_CurrVersNaviFile (p_path_Data, l_currVersNaviFile);
  }
  else if (SOURCE_CONTINENT == l_MediaType)
  {
    l_script_status = GetUpgDVD_CurrVersNaviFile (p_path_Data, l_currVersNaviFile);
    if(OK == l_script_status)
    {
      //l_script_status = Get_DVD_CIDInfoList(&l_CID_List);

    }

    if(OK == l_script_status)
    {
      // Get CID informations for all countries present on DVD
      UPGLogMsg ("GetNumberOfCIDsOnMedia: ('%s')\n",
                 (int) l_currVersNaviFile, 0, 0, 0, 0, 0);
      UPGLogMsg ("media path: ('%s')\n",
                 (int) p_path_Data, 0, 0, 0, 0, 0);

      if (OK == ExtractFileInfos (p_path_Data, l_currVersNaviFile, l_DVD_Content))
      {
        *l_CID_List = l_DVD_Content->m_CID_list;
        if (NULL == *l_CID_List)
        {
          UPGLogMsg ("GetNumberOfCIDsOnMedia: ERROR when extracting infos from '%s'\n", (int) l_filename, 0, 0, 0, 0, 0);
          l_script_status = ERROR ;
        }
        else
        {
          *l_CID_Count = l_DVD_Content->m_DVD_CID_number;
          UPGLogMsg ("GetNumberOfCIDsOnMedia:\n", 0, 0, 0, 0, 0, 0);
          UPGLogMsg ("- l_CID_Count               : %d\n",   *l_CID_Count, 0, 0, 0, 0, 0);
          UPGLogMsg ("- l_DVD_Content.m_DVD_title : '%s'\n", (char*) l_DVD_Content->m_DVD_title, 0, 0, 0, 0, 0);
        }
      }
      else
      {
        UPGLogMsg ("GetNumberOfCIDsOnMedia: ERROR when extracting infos from '%s'\n", (int) l_filename, 0, 0, 0, 0, 0);
        l_script_status = ERROR ;
      }
    }

  }


  return (l_script_status);
}



STATUS GetContinentOnMedia (char        *p_pPath,
                            t_continent *p_pCD_Continent)
{
  char    l_pDummy[MAX_CID_LENGTH];
  STATUS  l_Ret = OK;

  if (ERROR == GetNavigationValue (p_pPath, "CID", l_pDummy))
  {
    // Check if Source is DVD by reading the CONTINENT_ID
    if (ERROR == GetNavigationValue (p_pPath, "CONTINENT_ID", l_pDummy))
    {
      UPGLogMsg ("GetNavigationValue error on %s\n", (int) p_pPath, 0, 0, 0, 0, 0);
      l_Ret = ERROR;
    }
    else
    {
      *p_pCD_Continent = (t_continent) atoi (l_pDummy);
    }
  }
  else
  {
    // CONTINENT_ID = convert (country)
    *p_pCD_Continent = GetContinent (atoi (l_pDummy));
  }

  return (l_Ret);
}



STATUS GetCIDAndVersionDVDOnMedia  (t_CID_List *p_CID_List,
                                    char       *p_pCID,
                                    char       *p_pCDVerHigh,
                                    char       *p_pCDVerLow)
{
  strcpy (p_pCID,       p_CID_List->m_pCID);
  strcpy (p_pCDVerHigh, p_CID_List->m_pCDVersion);
  strcpy (p_pCDVerLow,  p_CID_List->m_pCDRelease);

  UPGLogMsg ("GetVersionDVDOnMedia: CID : '%s' - Ver : '%s.%s'\n",
                p_pCID, p_pCDVerHigh, p_pCDVerLow, 0, 0, 0);

  return (OK);
}


//! STATUS GetVersionCDOnMedia
//!
//! @param char *p_pSourceDrive   [in]
//! @param char *p_pCID           [in]
//! @param char *p_pKeyFileName   [in]
//! @param char *p_pRT3VerHigh    [out]
//! @param char *p_pRT3VerLow     [out]
//!
//! @return OK or ERROR

STATUS GetVersionCDOnMedia  (char *p_pSourceDrive,
                             char *p_pCID,
                             char *p_pKeyFileName,
                             char *p_pCDVerHigh,
                             char *p_pCDVerLow)
{
  char    l_pPath[MAX_PATH_LENGTH];
  STATUS  l_ret = OK;

  strcpy (l_pPath, p_pKeyFileName);

  if (OK == GetNavigationValue (l_pPath, "CID", p_pCID))
  {
    strcpy (l_pPath, p_pSourceDrive);
    strcat (l_pPath, "/MAPPE/");
    strcat (l_pPath, p_pCID);
    strcat (l_pPath, KEY_FILE_NAV);
    strcat (l_pPath, ".INF");
    if (ERROR == GetNavigationValue (l_pPath, "VERSION", p_pCDVerHigh))
    {
      UPGLogMsg ("GetVersionCDOnMedia: GetNavigationValue VERSION error on '%s'\n", l_pPath, 0, 0, 0, 0, 0);
      l_ret = ERROR;
    }
    else if (ERROR == GetNavigationValue (l_pPath, "RELEASE", p_pCDVerLow))
    {
      UPGLogMsg ("GetVersionCDOnMedia: RELEASE error on '%s'\n", l_pPath, 0, 0, 0, 0, 0);
      l_ret = ERROR;
    }
    else
    {
      UPGLogMsg ("GetVersionCDOnMedia:  CID : '%s' - Ver : '%s.%s'\n",
                  p_pCID, p_pCDVerHigh, p_pCDVerLow, 0, 0, 0);
    }
  }
  else
  {
    UPGLogMsg ("GetVersionCDOnMedia: RELEASE error on '%s'\n", l_pPath, 0, 0, 0, 0, 0);
    l_ret = ERROR;
  }

  return (l_ret);
}



//! void GetVersionOnProduct
//!
//! @param char *p_pTargetDrive   [in]
//! @param char *p_pCID           [in]
//! @param char *p_pRT3VerHigh    [out]
//! @param char *p_pRT3VerLow     [out]
//!
//! @return none
void GetVersionOnProduct (char *p_pTargetDrive,
                          char *p_pCID,
                          char *p_pRTVerHigh,
                          char *p_pRTVerLow)
{
  char l_pPath[MAX_PATH_LENGTH];

  //Get the DB version on RT3
  strcpy (l_pPath, p_pTargetDrive);
  strcat (l_pPath, "/MAPPE/");
  strcat (l_pPath, p_pCID);
  strcat (l_pPath, KEY_FILE_NAV);
  strcat (l_pPath, ".INF");

  if (ERROR == GetNavigationValue (l_pPath, "VERSION", p_pRTVerHigh))
  {
    strcpy (p_pRTVerHigh, "0");
    strcpy (p_pRTVerLow, "0");
  }
  else if (ERROR == GetNavigationValue (l_pPath, "RELEASE", p_pRTVerLow))
  {
    strcpy (p_pRTVerLow, "0");
  }

  UPGLogMsg ("GetVersionOnProduct:  CID : '%s' - Ver : '%s.%s'\n",
              p_pCID, p_pRTVerHigh, p_pRTVerLow, 0, 0, 0);
}



//! BOOL CheckProductVsMediaVersions
//!
//! @param char *l_pRT3VerHigh [in]
//! @param char *l_pRT3VerLow  [in]
//! @param char *l_pCDVerHigh  [in]
//! @param char *l_pCDVerLow   [in]
//
//! @return TRUE if upgrade required, else FALSE.
BOOL CheckProductVsCDMediaVersions (char *p_pRTVerHigh,
                                    char *p_pRTVerLow,
                                    char *p_pCDVerHigh,
                                    char *p_pCDVerLow)
{
  BOOL l_Bret = FALSE;

  if (atoi (p_pCDVerHigh) > atoi (p_pRTVerHigh))
  {
    UPGLogMsg ("UpgRequired = TRUE : p_pCDVerHigh (%d) > p_pRTVerHigh (%d)\n",
                atoi (p_pCDVerHigh), atoi (p_pRTVerHigh), 0, 0, 0, 0);
    l_Bret = TRUE;
  }
  else if (    (atoi (p_pCDVerHigh) == atoi (p_pRTVerHigh))
            && (atoi (p_pCDVerLow)  >= atoi (p_pRTVerLow))
          )
  {
    UPGLogMsg ("UpgRequired = TRUE : p_pCDVerLow (%d) >= p_pRTVerLow (%d)\n",
                atoi (p_pCDVerLow), atoi (p_pRTVerLow), 0, 0, 0, 0);
    l_Bret = TRUE;
  }
  else
  {
    UPGLogMsg ("UpgRequired = FLSE : p_pCDVerLow (%d) >= p_pRTVerLow (%d)\n",
                atoi (p_pCDVerLow), atoi (p_pRTVerLow), 0, 0, 0, 0);
  }

  return (l_Bret);
}



//! UPG_CID_STATE CheckProductVsDVDMediaVersions
//!
//! @param char *l_pRT3VerHigh   [in]
//! @param char *l_pRT3VerLow    [in]
//! @param char *l_pCDVerHigh    [in]
//! @param char *l_pCDVerLow     [in]
//!
//! @return NEWER, OLDER ou UP_TO_DATE
UPG_CID_STATE CheckProductVsDVDMediaVersions (char *p_pRT3VerHigh,
                                              char *p_pRT3VerLow,
                                              char *p_pCDVerHigh,
                                              char *p_pCDVerLow)
{
  UPG_CID_STATE l_RetState = NEWER;

  if (     (atoi (p_pCDVerHigh) > atoi (p_pRT3VerHigh))
       ||  (    (atoi (p_pCDVerHigh) == atoi (p_pRT3VerHigh))
             && (atoi (p_pCDVerLow)   > atoi (p_pRT3VerLow))
           )
     )
  {
    l_RetState = OLDER;
  }
  else if (    (atoi (p_pCDVerHigh) == atoi (p_pRT3VerHigh))
            && (atoi (p_pCDVerLow)  == atoi (p_pRT3VerLow))
          )
  {
    l_RetState = UP_TO_DATE;
  }

  return (l_RetState);
}


//! int DVDMakeCountriesListToUpgrade
//!
//! @param t_DVD_Content *p_pDVD_Content     [in]
//! @param int            p_count_real_list  [in]
//! @param t_CID_List    *p_pCID_real_List   [out]
//!
//! @return Number of elements in the p_pCID_real_List
int DVDMakeCountriesListToUpgrade (t_DVD_Content *p_pDVD_Content,
                                   int            p_count_real_list,
                                   t_CID_List    *p_pCID_real_List)
{
  int l_count_reduced = 0;
  int l_count         = 0;

  // Fill DVD list with only UpToDate or Older countries
  l_count_reduced= 0;
  for (l_count=0; l_count<p_pDVD_Content->m_DVD_CID_number; l_count++)
  {
    if (NEWER != p_pDVD_Content->m_CID_list[l_count].m_state)
    {
      strcpy (p_pCID_real_List[l_count_reduced].m_pCID,
              p_pDVD_Content->m_CID_list[l_count].m_pCID);
      strcpy (p_pCID_real_List[l_count_reduced].m_pCDVersion,
              p_pDVD_Content->m_CID_list[l_count].m_pCDVersion) ;
      strcpy (p_pCID_real_List[l_count_reduced].m_pCDRelease,
              p_pDVD_Content->m_CID_list[l_count].m_pCDRelease) ;
      strcpy (p_pCID_real_List[l_count_reduced].m_pHDDVersion, "0");
      strcpy (p_pCID_real_List[l_count_reduced].m_pHDDRelease, "0") ;
      p_pCID_real_List[l_count_reduced].m_state =
              p_pDVD_Content->m_CID_list[l_count].m_state;
      strcpy (p_pCID_real_List[l_count_reduced].m_CID_name,
              p_pDVD_Content->m_CID_list[l_count].m_CID_name) ;
      p_pCID_real_List[l_count_reduced].m_cid_size =
              p_pDVD_Content->m_CID_list[l_count].m_cid_size;

      l_count_reduced++;

      if (l_count_reduced > p_count_real_list)
      {
        l_count_reduced = -1;
        break;
      }
    }
  }

  return (l_count_reduced);
}



//! STATUS DVDCheckSpaceAndConfirmUpgrade
//!
//! @param char          *p_pSourceDrive            [in]
//! @param char          *p_pTargetDrive            [in]
//! @param char          *p_user_country_CID_choice [in]
//! @param char          *p_user_country_name_choice[in/out]
//! @param int            p_count_real_list         [in]
//! @param BOOL           p_bUpgFormatRequired      [in]
//! @param t_DVD_Content *p_reduced_DVD_Content     [in/out]
//! @param BOOL          *p_bUpgRequired            [in/out]
//! @param t_TYPE_DEVICE  p_MediaInfoType           [in]
//!
//! @return ERROR if Check error
STATUS DVDCheckSpaceAndConfirmUpgrade (char          *p_pSourceDrive,
                                       char          *p_pTargetDrive,
                                       char          *p_user_country_CID_choice,
                                       char          *p_user_country_name_choice,
                                       int            p_count_real_list,
                                       BOOL           p_bUpgFormatRequired,
                                       t_DVD_Content *p_reduced_DVD_Content,
                                       BOOL          *p_pbUpgRequired,
                                       t_TYPE_DEVICE  p_MediaInfoType)
{
  STATUS l_script_status = OK;
  int    l_ichoice = 0;
  int    l_ifound = 0;
  int    l_i = 0;
  int    l_value_size = 0 ;
  char  *l_indx1 = NULL;
  char  *l_indx2 = NULL;
  char   l_Name[MAX_VERSION_LENGTH];
  BOOL   l_Continue = TRUE;
  long long      l_dummy = 0;
  unsigned long  l_size_to_install = 0;
  unsigned long  l_free_space_inMB = 0;
  UPG_CID_STATE *l_pstate;
  int            l_FoundCountries = 0;
  t_UPGCIDSize  *l_pCountries = NULL;
  int            l_return = OK;
  unsigned long  l_supposed_new_space_after_remove = 0;
  int            l_count = 0;
  unsigned long  l_dummy_size;
  char           l_sDummySize[150];
  unsigned long  l_dummy_EPC_size;
  char           l_sDummyEPC_Size[150];
  char           l_sPath[150];

  if (p_count_real_list<=0)
  {
    *p_pbUpgRequired = FALSE;
    return (ERROR);
  }

  l_pstate = (UPG_CID_STATE *) malloc (sizeof (UPG_CID_STATE) * (p_count_real_list+1));
  if (NULL == l_pstate)
  {
    *p_pbUpgRequired = FALSE;
    return (ERROR);
  }

  // Push values...
  for (l_i=0; l_i<p_count_real_list; l_i ++)
  {
    l_pstate[l_i] = p_reduced_DVD_Content->m_CID_list[l_i].m_state;
  }

  if (0 == strcmp (p_user_country_CID_choice, ALL_COUNTRY))
  {
    UPGLogMsg ("DVDCheckSpceAndConfUpgrade: p_user_country_name_choice = ALL_COUNTRY\n", 0, 0, 0, 0, 0, 0);
    for (l_i=0; l_i<p_count_real_list; l_i ++)
    {
      if (    (UP_TO_DATE == p_reduced_DVD_Content->m_CID_list[l_i].m_state)
           || (OLDER      == p_reduced_DVD_Content->m_CID_list[l_i].m_state))
      {
        l_size_to_install += (p_reduced_DVD_Content->m_CID_list[l_i].m_cid_size)/(1024*1024)+1;

        UPGLogMsg ("DVDChckSpceAndConfUpgrade: UPGRADE USER CHOICE: CID=%s \n", (char*) p_reduced_DVD_Content->m_CID_list[l_i].m_pCID, 0, 0, 0, 0, 0);
        p_reduced_DVD_Content->m_CID_list[l_i].m_state = TO_UPDATE ;
        UPGLogMsg ("DVDChckSpceAndConfUpgrade: l_reduced_DVD_Content[l_i].m_state = %d - TO_UPDATE \n", p_reduced_DVD_Content->m_CID_list[l_i].m_state, 0, 0, 0, 0, 0);
      }
    }
  }
  else
  {
    UPGLogMsg ("DVDChckSpceAndConfUpgrade: l_NumberOfCountriesToUpgrade = %d \n", p_count_real_list, 0, 0, 0, 0, 0);
    l_ifound = FALSE;
    l_ichoice = 0;
    for (l_i=0; (l_i<p_count_real_list) && (FALSE==l_ifound); l_i++)
    {
      UPGLogMsg ("DVDChckSpceAndConfUpgrade: l_reduced_DVD_Content[%d].m_pCID='%s'\n",
                  l_i,
                  (char*) p_reduced_DVD_Content->m_CID_list[l_i].m_pCID,
                  0, 0, 0, 0);
      if (0 == strcmp (p_user_country_CID_choice,
                       p_reduced_DVD_Content->m_CID_list[l_i].m_pCID))
      {
        l_size_to_install = (p_reduced_DVD_Content->m_CID_list[l_i].m_cid_size)/(1024*1024)+1;
        l_ichoice = l_i;
        l_ifound = TRUE;
      }
    }

    if (TRUE == l_ifound)
    {
      if (    (UP_TO_DATE == p_reduced_DVD_Content->m_CID_list[l_ichoice].m_state)
           || (OLDER      == p_reduced_DVD_Content->m_CID_list[l_ichoice].m_state))
      {
        UPGLogMsg ("DVDChckSpceAndConfUpgrade: UPGRADE USER CHOICE: CID=%s \n", (char*) p_reduced_DVD_Content->m_CID_list[l_ichoice].m_pCID, 0, 0, 0, 0, 0);
        p_reduced_DVD_Content->m_CID_list[l_ichoice].m_state = TO_UPDATE ;
        UPGLogMsg ("DVDChckSpceAndConfUpgrade: l_reduced_DVD_Content[%d].m_state = %d - TO_UPDATE\n", l_i, p_reduced_DVD_Content->m_CID_list[l_ichoice].m_state, 0, 0, 0, 0);

        //delete state from l_user_country_name_choice
        UPGLogMsg ("DVDChckSpceAndConfUpgrade: Extract CID version \n", 0, 0, 0, 0, 0, 0);
        l_indx1 = p_user_country_name_choice ;
        l_indx2 = strstr (l_indx1, "(");
        if (NULL != l_indx2)
        {
          l_value_size = l_indx2 - l_indx1 ;
          strncpy (l_Name, l_indx1, l_value_size);
          l_Name[l_value_size] = '\0';
          UPGLogMsg ("DVDChckSpceAndConfUpgrade: l_Name= %s\n", (int) l_Name, 0, 0, 0, 0, 0);
          strcpy (p_user_country_name_choice, l_Name);
        }
        else
        {
          UPGLogMsg ("DVDChckSpceAndConfUpgrade: '(State)' not found in '%s'\n", p_user_country_name_choice, 0, 0, 0, 0, 0);
        }
      }
      else
      {
        UPGLogMsg ("DVDChckSpceAndConfUpgrade: l_reduced_DVD_Content[%d].m_state = %d - NOT TO_UPDATE \n", l_i, p_reduced_DVD_Content->m_CID_list[l_i].m_state, 0, 0, 0, 0);
      }
    }
    else
    {
      UPGLogMsg ("DVDChckSpceAndConfUpgrade: Not found l_i = %d\n", l_i, 0, 0, 0, 0, 0);
    }
  }

  UPGLogMsg ("DVDChckSpceAndConfUpgrade: l_user_country_name_choice = %s\n", p_user_country_name_choice, 0, 0, 0, 0, 0);

  // If not Format required
  if (FALSE == p_bUpgFormatRequired)
  {
    l_script_status = UPGGetFreeSpaceOnPartition (p_pTargetDrive, &l_free_space_inMB);
    if (OK == l_script_status)
    {
      if (l_free_space_inMB >= l_size_to_install)
      {
        // Ask confirmation to install (no size management needed).
        *p_pbUpgRequired = FALSE;
        ShowNavigationExtendedQuestionScreen (MMI_UPG_DISPLAY_ASK_MOTOR,
                                              p_user_country_name_choice,
                                              p_pbUpgRequired);
        if (FALSE == FileExists(p_pSourceDrive))
        {
          *p_pbUpgRequired = FALSE;
        }
      }
      else
      {
        while (TRUE == l_Continue)
        {
          l_script_status = UPGGetFreeSpaceOnPartition (p_pTargetDrive, &l_free_space_inMB);
          if (OK == l_script_status)
          {
            l_free_space_inMB = (l_free_space_inMB * FREE_SPACE_RATIO) / 100;
            UPGLogMsg ("DVDChckSpceAndConfUpgrade: %d %% of l_free_space = %ld Mb\n", FREE_SPACE_RATIO, l_free_space_inMB, 0, 0, 0, 0);
            UPGLogMsg ("DVDChckSpceAndConfUpgrade: Size to install = %d Mb\n", l_size_to_install, 0, 0, 0, 0, 0);

            if (l_size_to_install >= l_free_space_inMB)
            {
              l_script_status = UPGGetHddCountries (&l_FoundCountries, &l_pCountries);

              // If the country is present on the product, we search to deleting it
              // to restore the space.
              l_supposed_new_space_after_remove = 0;
              for (l_count; l_count<l_FoundCountries; l_count++)
              {
                /*    To delete into next version (>1.1.22) if works

                //{CID}/CD_VER_NAV.INF
                strcpy (l_sPath,"/SDH/MAPPE/");
                strcat (l_sPath,l_pCountries[l_count].pCID);
                strcat (l_sPath,"/CD_VER.NAV.INF");
                UPGLogMsg ("GetCIDSize ('%s')\n", l_sPath, 0, 0, 0, 0, 0);
                GetCIDSize (l_sPath, l_sDummySize,&l_dummy_size);
                l_pCountries[l_count].CIDSize = l_dummy_size;
                UPGLogMsg ("DVDCheckSpaceAndConfirmUpgrade:l_pCountries[%d] = %ld bytes\n",
                  l_i, l_pCountries[l_i].CIDSize, 0, 0, 0, 0);


                //{CID}/{CID}_EPC.BIN.inf
                strcpy (l_sPath,"/SDH/MAPPE/");
                strcat (l_sPath,l_pCountries[l_count].pCID);
                strcat (l_sPath,"/");
                strcat (l_sPath,l_pCountries[l_count].pCID);
                strcat (l_sPath,"_EPC.BIN.inf");
                UPGLogMsg ("GetCIDSize of ('%s') if EXIST \n", l_sPath, 0, 0, 0, 0, 0);
                if (TRUE == FileExists(l_sPath))
                {
                  UPGLogMsg ("GetCIDSize ('%s')\n", l_sPath, 0, 0, 0, 0, 0);
                  GetCIDSize (l_sPath, l_sDummyEPC_Size,&l_dummy_EPC_size);
                  l_pCountries[l_count].CIDSize += l_dummy_EPC_size;
                  UPGLogMsg ("DVDCheckSpaceAndConfirmUpgrade:l_pCountries[%d] = %ld bytes\n",
                             l_i, l_pCountries[l_i].CIDSize, 0, 0, 0, 0);

                }
*/

                //"CD_VER_NAV.INF" and "{CID}_EPC.BIN.inf"
                if (OK == l_script_status)
                {

                  if (ERROR == GetCorrectCountry_Size ("/SDH",
                                                       l_pCountries[l_count].pCID,
                                                       &l_dummy_size))
                  {
                    UPGLogMsg ("[ERROR] - DVDChckSpceAndConfUpgrade :  GetCorrectCountry_Size() Failed for '/SDH'\n",
                               0, 0, 0, 0, 0, 0);
                  }
                  else
                  {
                    l_pCountries[l_count].CIDSize = l_dummy_size;
                    UPGLogMsg ("DVDChckSpceAndConfUpgrade:l_pCountries[%d] = %ld Mb\n",
                               l_count, l_pCountries[l_count].CIDSize, 0, 0, 0, 0);
                  }

                }



                // Country is on the list ?
                if (l_pCountries[l_count].CID == atoi(p_user_country_CID_choice))
                {
                  /*    To delete into next version (>1.1.22) if works
                  l_supposed_new_space_after_remove = (l_pCountries[l_count].CIDSize / (1024*1024)) + l_free_space_inMB;
                  */

                  l_supposed_new_space_after_remove = l_pCountries[l_count].CIDSize + l_free_space_inMB;

                  if (l_supposed_new_space_after_remove > l_size_to_install)
                  {
                    // Ask confirmation to install (no size management needed).
                    *p_pbUpgRequired = FALSE;
                    ShowNavigationExtendedQuestionScreen (MMI_UPG_DISPLAY_ASK_MOTOR_STOP_GUID,
                                        p_user_country_name_choice,
                                        p_pbUpgRequired);
                    if (TRUE == *p_pbUpgRequired)
                    {
                      if (FALSE == FileExists(p_pSourceDrive))
                      {
                        *p_pbUpgRequired = FALSE;
                      }
                      else
                      {
                        UPGStopGuidance ();
                        UPGXdeleteCountry (p_pTargetDrive, l_pCountries[l_count].pCID);
                        UPGCheckCarto ();

                        if (ERROR == UPGInformNavForNewMapAndWaitAck (p_MediaInfoType))
                        {
                          UPGLogMsg ("ERROR on UPGInformNavForNewMapAndWaitAck\n", 0, 0, 0, 0, 0, 0);
                          l_script_status = ERROR;
                        }
                      }
                    }
                    l_Continue = FALSE;
                  }
                  break;
                }
              }

              if (OK == l_script_status)
              {
                if (TRUE == l_Continue)
                {
                  if (1 == l_FoundCountries)
                  {
                    *p_pbUpgRequired = FALSE;
                    ShowNavigationExtendedQuestionScreen (MMI_UPG_DISPLAY_ASK_MOTOR_SUPPRESS_THE_CARD,
                                      p_user_country_name_choice,
                                      p_pbUpgRequired);
                    if (TRUE == *p_pbUpgRequired)
                    {
                      if (FALSE == FileExists(p_pSourceDrive))
                      {
                        *p_pbUpgRequired = FALSE;
                      }
                      else
                      {
                        UPGStopGuidance ();
                        UPGXdeleteCountry (p_pTargetDrive, l_pCountries[0].pCID);
                        UPGCheckCarto ();
                        if (ERROR == UPGInformNavForNewMapAndWaitAck (p_MediaInfoType))
                        {
                          UPGLogMsg ("ERROR on UPGInformNavForNewMapAndWaitAck\n", 0, 0, 0, 0, 0, 0);
                          l_script_status = ERROR;
                        }
                      }
                    }
                    l_Continue = FALSE;
                  }
                  else
                  {
                    *p_pbUpgRequired = FALSE;
                    ShowNavigationExtendedQuestionScreen (MMI_UPG_DISPLAY_ASK_MOTOR_SUPPRESS_A_CARD,
                                        p_user_country_name_choice,
                                        p_pbUpgRequired);
                    if (TRUE == *p_pbUpgRequired)
                    {
                      if (TRUE == FileExists(p_pSourceDrive))
                      {
                        l_return = UPGSizeManagement (p_pSourceDrive, p_pTargetDrive, l_size_to_install, p_pbUpgRequired, p_MediaInfoType);
                        if (ESCAPE == l_return)
                        {
                          l_script_status = OK;
                          l_Continue      = TRUE;
                        }
                        else if (ERROR == l_return)
                        {
                          l_script_status = ERROR;
                          l_Continue      = FALSE;
                        }
                        else
                        {
                          l_script_status = OK;
                          l_Continue      = FALSE;
                        }
                      }
                      else
                      {
                        l_script_status = ERROR;
                        l_Continue      = FALSE;
                      }
                    }
                    else
                    {
                      l_script_status = OK;
                      l_Continue      = FALSE;
                    }
                  }
                }
              }
              else
              {
                l_Continue        = FALSE;
                *p_pbUpgRequired  = FALSE;
              }
            }
            else
            {
              l_Continue        = FALSE;
              *p_pbUpgRequired  = TRUE;
            }
          }
          else
          {
            // ERROR on UPGGetFreeSpaceOnPartition
            l_Continue        = FALSE;
            *p_pbUpgRequired  = FALSE;
          }
        }
      }
    }
    else
    {
      UPGLogMsg ("ERROR on UPGGetFreeSpaceOnPartition\n",0,0,0,0,0,0);
      l_Continue       = FALSE;
      *p_pbUpgRequired = FALSE;
      l_script_status  = ERROR;
    }
  }
  else
  {
    if (TRUE == FileExists(p_pSourceDrive))
    {
      // Confirmation of Formating before update
      *p_pbUpgRequired = FALSE;
      ShowNavigationExtendedQuestionScreen (MMI_UPG_DISPLAY_ASK_MOTOR_STOP_GUID,
                                            p_user_country_name_choice,
                                            p_pbUpgRequired);
      if (FALSE == FileExists(p_pSourceDrive))
      {
        *p_pbUpgRequired = FALSE;
      }
    }
  }

  if (FALSE == FileExists(p_pSourceDrive))
  {
    l_script_status = OK;
    l_Continue      = FALSE;
    *p_pbUpgRequired = FALSE;
  }

  if (FALSE == *p_pbUpgRequired)
  {
    // Restore values...
    for (l_i=0; l_i<p_count_real_list; l_i ++)
    {
      p_reduced_DVD_Content->m_CID_list[l_i].m_state = l_pstate[l_i];
    }
  }

  if (NULL != l_pstate)
  {
    free (l_pstate);
  }

  return (l_script_status);
}

//! STATUS GetCIDSize
//!
//! @param char          *p_pPath        [in]
//! @param char          *p_psCIDSize    [out]
//! @param unsigned long *p_lCID_Size    [out]
//!
//! @return ERROR or OK
STATUS GetCIDSize (char *p_pPath, char *p_psCIDSize, unsigned long *p_lCID_Size)
{
  STATUS l_ret = OK;
  int    l_SD_BlockSize = 0;

  *p_lCID_Size = 0;
  strcpy (p_psCIDSize, "");

  l_SD_BlockSize = GetSDBlockSize();
  switch (l_SD_BlockSize)
  {
    case 32768:
    {
      if (ERROR == GetNavigationValue (p_pPath, "CID_SIZE_32", p_psCIDSize))
      {
        UPGLogMsg ("GetCIDSize: GetNavigationValue CID_SIZE_32 error on '%s'\n", p_pPath, 0, 0, 0, 0, 0);
        l_ret = ERROR;
      }
      break;
    }

    case 16384:
    {
      if (ERROR == GetNavigationValue (p_pPath, "CID_SIZE_16", p_psCIDSize))
      {
        UPGLogMsg ("GetCIDSize: GetNavigationValue CID_SIZE_16 error on '%s'\n", p_pPath, 0, 0, 0, 0, 0);
        l_ret = ERROR;
      }
      break;
    }

    case 8192:
    {
      if (ERROR == GetNavigationValue (p_pPath, "CID_SIZE_8", p_psCIDSize))
      {
        UPGLogMsg ("GetCIDSize: GetNavigationValue CID_SIZE_8 error on '%s'\n", p_pPath, 0, 0, 0, 0, 0);
        l_ret = ERROR;
      }
      break;
    }

    case 4096:
    {
      if (ERROR == GetNavigationValue (p_pPath, "CID_SIZE_4", p_psCIDSize))
      {
        UPGLogMsg ("GetCIDSize: GetNavigationValue CID_SIZE_4 error on '%s'\n", p_pPath, 0, 0, 0, 0, 0);
        l_ret = ERROR;
      }
      break;
    }

    default:
    {
      UPGLogMsg ("GetCIDSize: SD Block size (%ld) not managed!\n", (int)l_SD_BlockSize, 0, 0, 0, 0, 0);
      l_ret = ERROR;
      break;
    }
  }

  if (l_ret == ERROR)
  {
    l_ret = OK;
    if (ERROR == GetNavigationValue (p_pPath, "CID_SIZE", p_psCIDSize))
    {
      UPGLogMsg ("GetCIDSize: GetNavigationValue CID_SIZE error on '%s'\n", p_pPath, 0, 0, 0, 0, 0);
      l_ret = ERROR;
    }
  }

  if (OK == l_ret)
  {
    *p_lCID_Size = atol (p_psCIDSize);
    UPGLogMsg ("GetCIDSize:  l_pCD_SIZE = '%s' bytes, *p_pCID_Size = %d\n",
              p_psCIDSize, *p_lCID_Size , 0, 0, 0, 0);
  }

  return (l_ret);
}



//! STATUS GetCID_EPCSize
//!
//! @param char          *p_pPath        [in]
//! @param char          *p_psCIDSize    [out]
//! @param unsigned long *p_lCID_Size    [out]
//!
//! @return ERROR or OK
STATUS GetCID_EPCSize (char *p_pPath, char *p_psCIDSize, unsigned long *p_lCID_Size)
{
  STATUS l_ret = OK;
  int    l_SD_BlockSize = 0;

  *p_lCID_Size = 0;
  strcpy (p_psCIDSize, "");

  l_SD_BlockSize = GetSDBlockSize();
  switch (l_SD_BlockSize)
  {
    case 32768:
    {
      if (ERROR == GetNavigationValue (p_pPath, "SIZE_32", p_psCIDSize))
      {
        UPGLogMsg ("GetCID_EPCSize: GetNavigationValue SIZE_32 error on '%s'\n", p_pPath, 0, 0, 0, 0, 0);
        l_ret = ERROR;
      }
      break;
    }

    case 16384:
    {
      if (ERROR == GetNavigationValue (p_pPath, "SIZE_16", p_psCIDSize))
      {
        UPGLogMsg ("GetCID_EPCSize: GetNavigationValue SIZE_16 error on '%s'\n", p_pPath, 0, 0, 0, 0, 0);
        l_ret = ERROR;
      }
      break;
    }

    case 8192:
    {
      if (ERROR == GetNavigationValue (p_pPath, "SIZE_8", p_psCIDSize))
      {
        UPGLogMsg ("GetCID_EPCSize: GetNavigationValue SIZE_8 error on '%s'\n", p_pPath, 0, 0, 0, 0, 0);
        l_ret = ERROR;
      }
      break;
    }

    case 4096:
    {
      if (ERROR == GetNavigationValue (p_pPath, "SIZE_4", p_psCIDSize))
      {
        UPGLogMsg ("GetCID_EPCSize: GetNavigationValue SIZE_4 error on '%s'\n", p_pPath, 0, 0, 0, 0, 0);
        l_ret = ERROR;
      }
      break;
    }

    default:
    {
      UPGLogMsg ("GetCID_EPCSize: SD Block size (%ld) not managed!\n", (int)l_SD_BlockSize, 0, 0, 0, 0, 0);
      l_ret = ERROR;
      break;
    }
  }

  if (l_ret == ERROR)
  {
    l_ret = OK;
    if (ERROR == GetNavigationValue (p_pPath, "SIZE", p_psCIDSize))
    {
      UPGLogMsg ("GetCID_EPCSize: GetNavigationValue SIZE error on '%s'\n", p_pPath, 0, 0, 0, 0, 0);
      l_ret = ERROR;
    }
  }

  if (OK == l_ret)
  {
    *p_lCID_Size = atol (p_psCIDSize);
    UPGLogMsg ("GetCID_EPCSize:  l_pCD_SIZE = '%s' bytes, *p_pCID_Size = %d\n",
              p_psCIDSize, *p_lCID_Size , 0, 0, 0, 0);
  }

  return (l_ret);
}



//! STATUS CDCheckSpaceAndConfirmUpgrade
//!
//! @param char          *p_pSourceDrive        [in]
//! @param char          *p_pTargetDrive        [in]
//! @param char          *p_pKeyFileName        [in]
//! @param char          *p_pCDVerHigh          [in]
//! @param char          *p_pCDVerLow           [in]
//! @param BOOL          *p_bUpgRequired        [in/out]
//! @param t_TYPE_DEVICE  p_MediaInfoType       [in]
//! @param char          *p_user_country_name_choice [out]
//!
//! @return ERROR if Check error
STATUS CDCheckSpaceAndConfirmUpgrade (char          *p_pSourceDrive,
                                      char*          p_path_Data,
                                      char          *p_pTargetDrive,
                                      char          *p_pKeyFileName,
                                      char          *p_pCDVerHigh,
                                      char          *p_pCDVerLow,
                                      BOOL          *p_pbUpgRequired,
                                      t_TYPE_DEVICE  p_MediaInfoType,
                                      char          *p_user_country_name_choice)
{
  char l_pPath[MAX_PATH_LENGTH];
  char l_pPathInf[MAX_PATH_LENGTH];
  char l_pCD_NAME[MAX_INF_LINE_LENGTH];
  char l_pCD_CID[MAX_INF_LINE_LENGTH];
  char l_pCD_CID_SIZE[MAX_INF_LINE_LENGTH];
  char l_pCD_CID_EPC_SIZE[MAX_INF_LINE_LENGTH];
  char l_pCD_PROVIDER[MAX_INF_LINE_LENGTH];
  char l_pQuestionText[MAX_QUESTION_LENGTH];

  unsigned long   l_size_to_install = 0;
  unsigned long   l_size_of_EPC_to_install = 0;
  int             l_FoundCountries  = 0;
  t_UPGCIDSize   *l_pCountries      = NULL;
  unsigned long   l_free_space_inMB = 0;
  BOOL            l_Continue        = TRUE;
  STATUS          l_script_status   = OK;
  int             l_count           = 0;
  int             l_return          = 0;
  unsigned long   l_supposed_new_space_after_remove = 0;
  unsigned long   l_dummy_size;
  char            l_sDummySize[150];
  char            l_sPath[150];

  strcpy (p_user_country_name_choice, " ");

  strcpy (l_pPath, p_pKeyFileName);

  if (ERROR == GetNavigationValue (l_pPath, "CD_NAME", l_pCD_NAME))
  {
    UPGLogMsg ("GetNavigationValue CD_NAME error on %s\n", l_pPath, 0, 0, 0, 0, 0);
    l_script_status = ERROR;
  }
  else if (ERROR == GetNavigationValue (l_pPath, "CD_PROVIDER", l_pCD_PROVIDER))
  {
    UPGLogMsg ("GetNavigationValue CD_PROVIDER error on %s\n", l_pPath, 0, 0, 0, 0, 0);
    l_script_status = ERROR;
  }
  else if (ERROR == GetNavigationValue (l_pPath, "CID", l_pCD_CID))
  {
    UPGLogMsg ("GetNavigationValue CID error on %s\n", l_pPath, 0, 0, 0, 0, 0);
    l_script_status = ERROR;
  }

  //"CD_VER_NAV.INF" and "{CID}_EPC.BIN.inf"
  if (OK == l_script_status)
  {
    // For the caller...
    strcpy (p_user_country_name_choice, l_pCD_NAME);

    l_script_status = GetCorrectCountry_Size (p_path_Data,
                                              l_pCD_CID,
                                              &l_size_to_install);

    if (ERROR == l_script_status)
    {
      UPGLogMsg ("[ERROR] - CDCheckSpaceAndConfirmUpgrade :  GetCorrectCountry_Size() Failed for %s\n",
                 p_path_Data, 0, 0, 0, 0, 0);
    }

  }




/*    To delete into next version (>1.1.22) if works
  //CD_VER_NAV.INF
  if (OK == l_script_status)
  {
    // For the caller...
    strcpy (p_user_country_name_choice, l_pCD_NAME);

    strcpy (l_pPathInf, p_path_Data);
    strcat (l_pPathInf, "/MAPPE/");
    strcat (l_pPathInf, l_pCD_CID);
    strcat (l_pPathInf, KEY_FILE_NAV);
    strcat (l_pPathInf, ".INF");

    if (ERROR == GetCIDSize (l_pPathInf, l_pCD_CID_SIZE, &l_size_to_install))
    {

      l_script_status = ERROR;
    }
    else
    {
      l_size_to_install = (l_size_to_install/(1024*1024))+1;
      UPGLogMsg ("CDCheckSpaceAndConfirmUpgrade:  l_size_to_install = %d Mb\n", l_size_to_install, 0, 0, 0, 0, 0);
    }
  }
*/


/*    To delete into next version (>1.1.22) if works
  //{CID}_EPC.BIN.inf
  if (OK == l_script_status)
  {
    // For the caller...
    strcpy (p_user_country_name_choice, l_pCD_NAME);

    strcpy (l_pPathInf, p_path_Data);
    strcat (l_pPathInf, "/MAPPE/");
    strcat (l_pPathInf, l_pCD_CID);
    strcat (l_pPathInf, "/");
    strcat (l_pPathInf, l_pCD_CID);
    strcat (l_pPathInf, "_EPC.BIN.inf");

    //check first if the file exist
    if (TRUE == FileExists(l_pPathInf))
    {
      if (ERROR == GetCID_EPCSize (l_pPathInf, l_pCD_CID_EPC_SIZE, &l_size_of_EPC_to_install))
      {
        UPGLogMsg ("GetCIDSize error on %s\n", l_pPathInf, 0, 0, 0, 0, 0);
        l_script_status = ERROR;
      }
      else
      {
        l_size_of_EPC_to_install = (l_size_of_EPC_to_install/(1024*1024))+1;
        UPGLogMsg ("CDCheckSpaceAndConfirmUpgrade:  l_size_of_EPC_to_install = %d Mb\n", l_size_of_EPC_to_install, 0, 0, 0, 0, 0);
      }
    }
  }
*/

  if (OK == l_script_status)
  {
    UPGLogMsg ("CDCheckSpaceAndConfirmUpgrade:  total size to install = %d Mb\n",
               l_size_to_install, 0, 0, 0, 0, 0);

    l_script_status = UPGGetFreeSpaceOnPartition (p_pTargetDrive, &l_free_space_inMB);
    if (OK == l_script_status)
    {
      if (l_free_space_inMB >= (l_size_to_install+l_size_of_EPC_to_install))
      {
        // Ask confirmation to install (no size management needed).
        *p_pbUpgRequired = FALSE;
        ShowNavigationExtendedQuestionScreen (MMI_UPG_DISPLAY_ASK_MOTOR,
                                              l_pCD_NAME,
                                              p_pbUpgRequired);
        UPGLogMsg ("p_pSourceDrive = %s\n", (int)p_pSourceDrive, 0, 0, 0, 0, 0);
        if (FALSE == FileExists(p_pSourceDrive))
        {
          *p_pbUpgRequired = FALSE;
          UPGLogMsg ("upgrade not require  FALSE\n", 0, 0, 0, 0, 0, 0);
        }
      }
      else
      {
        while (TRUE == l_Continue)
        {
          l_script_status = UPGGetFreeSpaceOnPartition (p_pTargetDrive, &l_free_space_inMB);
          if (OK == l_script_status)
          {
            l_free_space_inMB = (l_free_space_inMB * FREE_SPACE_RATIO) / 100;
            UPGLogMsg ("CDCheckSpaceAndConfirmUpgrade: %d %% of l_free_space = %ld Mb\n", FREE_SPACE_RATIO, l_free_space_inMB, 0, 0, 0, 0);
            UPGLogMsg ("CDCheckSpaceAndConfirmUpgrade: Size to install = %ld Mb\n",
                        (l_size_to_install+l_size_of_EPC_to_install), 0, 0, 0, 0, 0);
            if ((l_size_to_install+l_size_of_EPC_to_install) >= l_free_space_inMB)
            {
              l_script_status = UPGGetHddCountries (&l_FoundCountries, &l_pCountries);

              // If the country is present on the product, we search to deleting it
              // to restore the space.
              l_supposed_new_space_after_remove = 0;
              for (l_count; l_count<l_FoundCountries; l_count++)
              {

/*    To delete into next version (>1.1.22) if works

                //CD_VER_NAV.INF
                strcpy (l_sPath,"/SDH/MAPPE/");
                strcat (l_sPath,l_pCountries[l_count].pCID);
                strcat (l_sPath,"/CD_VER.NAV.INF");
                UPGLogMsg ("GetCIDSize ('%s')\n", l_sPath, 0, 0, 0, 0, 0);
                GetCIDSize (l_sPath, l_sDummySize,&l_dummy_size);
                l_pCountries[l_count].CIDSize = l_dummy_size;

                //{CID}_EPC.BIN.inf
                strcpy (l_sPath,"/SDH/MAPPE/");
                strcat (l_sPath,l_pCountries[l_count].pCID);
                strcat (l_sPath,"/");
                strcat (l_sPath,l_pCountries[l_count].pCID);
                strcat (l_sPath,"_EPC.BIN.inf");

                if (TRUE == FileExists(l_sPath))
                {
                  UPGLogMsg ("GetCIDSize of EPC_BIN ('%s')\n", l_sPath, 0, 0, 0, 0, 0);
                  GetCIDSize (l_sPath, l_sDummySize,&l_dummy_size);
                  l_pCountries[l_count].CIDSize += l_dummy_size;
                }

                UPGLogMsg ("CDCheckSpaceAndConfirmUpgrade:l_pCountries[%d] = %ld bytes\n",
                  l_count, l_pCountries[l_count].CIDSize, 0, 0, 0, 0);
*/



                //"CD_VER_NAV.INF" and "{CID}_EPC.BIN.inf"
                if (OK == l_script_status)
                {

                  if (ERROR == GetCorrectCountry_Size ("/SDH",
                                                      l_pCountries[l_count].pCID,
                                                      &l_dummy_size))
                  {
                    UPGLogMsg ("[ERROR] - CDCheckSpaceAndConfirmUpgrade :  GetCorrectCountry_Size() Failed for '/SDH'\n",
                               0, 0, 0, 0, 0, 0);
                  }
                  else
                  {
                    l_pCountries[l_count].CIDSize = l_dummy_size;
                    UPGLogMsg ("CDCheckSpaceAndConfirmUpgrade:l_pCountries[%d] = %ld Mb\n",
                  l_count, l_pCountries[l_count].CIDSize, 0, 0, 0, 0);
                  }

                }


                // Country is on the list ?
                if (l_pCountries[l_count].CID == atoi(l_pCD_CID))
                {
                  /*    To delete into next version (>1.1.22) if works
                  l_supposed_new_space_after_remove = (l_pCountries[l_count].CIDSize / (1024*1024)) + l_free_space_inMB;
                  */
                  l_supposed_new_space_after_remove = l_pCountries[l_count].CIDSize + l_free_space_inMB;

                  if (l_supposed_new_space_after_remove > (l_size_to_install+l_size_of_EPC_to_install))
                  {
                    // Ask confirmation to install (no size management needed).
                    *p_pbUpgRequired = FALSE;
                    ShowNavigationExtendedQuestionScreen (MMI_UPG_DISPLAY_ASK_MOTOR_STOP_GUID,
                                                          l_pCD_NAME,
                                                          p_pbUpgRequired);
                    if (TRUE == *p_pbUpgRequired)
                    {
                      if (FALSE == FileExists(p_pSourceDrive))
                      {
                        *p_pbUpgRequired = FALSE;
                      }
                      else
                      {
                        UPGStopGuidance ();
                        UPGXdeleteCountry (p_pTargetDrive, l_pCountries[l_count].pCID);
                        UPGCheckCarto ();

                        if (ERROR == UPGInformNavForNewMapAndWaitAck (p_MediaInfoType))
                        {
                          UPGLogMsg ("ERROR on UPGInformNavForNewMapAndWaitAck\n", 0, 0, 0, 0, 0, 0);
                          l_script_status = ERROR;
                        }
                      }
                    }
                    l_Continue = FALSE;
                  }
                  break;
                }
              }

              if (OK == l_script_status)
              {
                if (TRUE == l_Continue)
                {
                  if (1 == l_FoundCountries)
                  {
                    *p_pbUpgRequired = FALSE;
                    ShowNavigationExtendedQuestionScreen (MMI_UPG_DISPLAY_ASK_MOTOR_SUPPRESS_THE_CARD,
                                                          l_pCD_NAME,
                                                          p_pbUpgRequired);
                    if (TRUE == *p_pbUpgRequired)
                    {
                      if (FALSE == FileExists(p_pSourceDrive))
                      {
                        *p_pbUpgRequired = FALSE;
                      }
                      else
                      {
                        UPGStopGuidance ();
                        UPGXdeleteCountry (p_pTargetDrive, l_pCountries[0].pCID);
                        UPGCheckCarto ();

                        if (ERROR == UPGInformNavForNewMapAndWaitAck (p_MediaInfoType))
                        {
                          UPGLogMsg ("ERROR on UPGInformNavForNewMapAndWaitAck\n", 0, 0, 0, 0, 0, 0);
                          l_script_status = ERROR;
                          l_Continue = FALSE;
                        }
                      }
                    }
                    l_Continue = FALSE;
                  }
                  else
                  {
                    *p_pbUpgRequired = FALSE;
                    ShowNavigationExtendedQuestionScreen (MMI_UPG_DISPLAY_ASK_MOTOR_SUPPRESS_A_CARD,
                                                          l_pCD_NAME,
                                                          p_pbUpgRequired);
                    if (TRUE == *p_pbUpgRequired)
                    {
                      if (TRUE == FileExists(p_pSourceDrive))
                      {
                        l_return = UPGSizeManagement (p_pSourceDrive, p_pTargetDrive, (l_size_to_install+l_size_of_EPC_to_install), p_pbUpgRequired, p_MediaInfoType);

                        if (ESCAPE == l_return)
                        {
                          l_script_status = OK;
                          l_Continue      = TRUE;
                        }
                        else if (ERROR == l_return)
                        {
                          l_script_status = ERROR;
                          l_Continue      = FALSE;
                        }
                        else
                        {
                          l_script_status = OK;
                          l_Continue      = FALSE;
                        }
                      }
                      else
                      {
                        l_script_status   = OK;
                        l_Continue        = FALSE;
                        *p_pbUpgRequired  = FALSE;
                      }
                    }
                    else
                    {
                      l_script_status = OK;
                      l_Continue      = FALSE;
                    }
                  }
                }
              }
              else
              {
                l_Continue        = FALSE;
                *p_pbUpgRequired  = FALSE;
              }
            }
            else
            {
              l_Continue        = FALSE;
              *p_pbUpgRequired  = TRUE;
            }
          }
          else
          {
            UPGLogMsg ("ERROR on UPGGetFreeSpaceOnPartition\n", 0, 0, 0, 0, 0, 0);
            l_Continue        = FALSE;
            *p_pbUpgRequired  = FALSE;
            l_script_status   = ERROR;
          }
        }
      }
    }
    else
    {
      UPGLogMsg ("ERROR on UPGGetFreeSpaceOnPartition\n", 0, 0, 0, 0, 0, 0);
      l_Continue        = FALSE;
      *p_pbUpgRequired  = FALSE;
      l_script_status   = ERROR;
    }
  }

  return (l_script_status);
}



//! void RemoveUnwantedFiles
//!
void RemoveUnwantedFiles (void)
{
  UPGLogMsg ("Remove unwanted files +, Time = %ld \n", tickGet(), 0, 0, 0, 0, 0);

  xdelete ("/SDH/TEMP*.*");
  xdelete ("/SDH/TEMP*.*.*");
  xdelete ("/SDH/MAPPE/TEMP*");

  UPGDelete ("/SDH/SW_VER.DAT");
  UPGDelete ("/SDH/DVD_VER.NAV");
  UPGDelete ("/SDH/CD_VER.NAV");
  UPGDelete ("/SDH/CD_VER.LA");

  UPGLogMsg ("Remove unwanted files -, Time = %ld \n", tickGet(), 0, 0, 0, 0, 0);
}



//! BOOL DVDGetCIDFromReducedList
//!
//! @param t_DVD_Content *p_pReduced_DVD_Content    [in]
//! @param int            p_i                       [in]
//! @param char          *p_pCID                    [out]
//!
//! @return TRUE if l_pCID is valid
BOOL DVDGetCIDFromReducedList (t_DVD_Content *p_pReduced_DVD_Content,
                               int            p_i,
                               char          *p_pCID)
{
  BOOL l_bCIDtoUpdate = FALSE;

  strcpy (p_pCID, p_pReduced_DVD_Content->m_CID_list[p_i].m_pCID);

  if (    (0 != strlen (p_pCID))
       && (TO_UPDATE == p_pReduced_DVD_Content->m_CID_list[p_i].m_state)
     )
  {
    l_bCIDtoUpdate = TRUE;
  }
  else
  {
    l_bCIDtoUpdate = FALSE;
  }

  return (l_bCIDtoUpdate);
}



//! BOOL CheckAndSaveFileIfNeeded
//!
//! @param char       *p_pFilePath       [in]
//! @param char       *p_pSourceDrive    [in]
//! @param char       *p_pTargetDrive    [in]
//!
//! @return TRUE if File is saved
BOOL CheckAndSaveFileIfNeeded (char *p_pFilePath,
                               char *p_pSourceDrive,
                               char *p_pTargetDrive)
{
  BOOL    l_bKeepDBFiles  = FALSE;
  STATUS  l_status        = OK;
  DIR*    l_pDir          = NULL;
  char    l_pSourcePath[MAX_PATH_LENGTH];
  char    l_pTargetPath[MAX_PATH_LENGTH];
  char    l_pDummyPath [MAX_PATH_LENGTH];

  char    l_pSourceVersion[MAX_VERSION_LENGTH];
  char    l_pTargetVersion[MAX_VERSION_LENGTH];

  strcpy (l_pSourcePath, p_pSourceDrive);
  strcat (l_pSourcePath, p_pFilePath);
  strcpy (l_pTargetPath, p_pTargetDrive);
  strcat (l_pTargetPath, p_pFilePath);

  // If the File on the target is more recent than the source, we save it...
  if (FALSE == UPGCheckIfVersionIsNewerThan (l_pSourcePath, l_pTargetPath))
  {
    l_bKeepDBFiles = TRUE;

    UPGLogMsg ("SaveFileIfNeeded: '%s' is UP TO date (%s -> %s)\n",
               (int) l_pTargetPath,
               (int) l_pTargetVersion, (int) l_pSourceVersion, 0, 0, 0);

    // Make Target path, and verify/construct the path...
    strcpy (l_pDummyPath, TEMP_DB_ZONE);
    // Create TEMP_DB_ZONE if not exist
    if (NULL == (l_pDir = opendir (l_pDummyPath)))
    {
      UPGMKDir (l_pDummyPath);
    }
    else
    {
      // If exist, empty it!
      xdelete (l_pDummyPath);
    }
    UPGLogMsg ("SaveFileIfNeeded: Save DB files on '%s'\n", (int)l_pDummyPath, 0, 0, 0, 0, 0);
    strcat (l_pDummyPath, p_pFilePath);

    UPGCopy (l_pTargetPath, l_pDummyPath, FALSE);
    UPGLogMsg ("SaveFileIfNeeded: Save '%s' to '%s' \n",
               (int) l_pTargetPath, (int) l_pDummyPath, 0, 0, 0, 0);

    strcat (l_pTargetPath, ".inf");
    strcat (l_pDummyPath, ".inf");

    UPGCopy (l_pTargetPath, l_pDummyPath, FALSE);
    UPGLogMsg ("SaveFileIfNeeded: Save '%s' to '%s' \n",
               (int) l_pTargetPath, (int) l_pDummyPath, 0, 0, 0, 0);
  }

  return (l_bKeepDBFiles);
}



//! STATUS CopyFileFromSrcDriveToTargDrive
//!
//! @param char       *p_pPathSourceFile [in]
//! @param char       *p_pPathTargetFile [in]
//! @param char       *p_pSourceDrive    [in]
//! @param char       *p_pTargetDrive    [in]
//!
//! @return OK or ERROR
STATUS CopyFileFromSrcDriveToTargDrive (char *p_pPathSourceFile,
                                        char *p_pPathTargetFile,
                                        char *p_pSourceDrive,
                                        char *p_pTargetDrive)
{
  char    l_pSourcePath [MAX_PATH_LENGTH];
  char    l_pTargetPath [MAX_PATH_LENGTH];
  STATUS  l_status = OK;
  int l_nb_trials = UPG_MAX_TRIALS;

  strcpy (l_pSourcePath, p_pSourceDrive);
  strcat (l_pSourcePath, p_pPathSourceFile);

  strcpy (l_pTargetPath, p_pTargetDrive);
  strcat (l_pTargetPath, p_pPathTargetFile);

  //l_status = UPGCopy (l_pSourcePath, l_pTargetPath, FALSE);
  if (OK == l_status)
  {
    UPGLogMsg ("Copy '%s' to '%s' : OK\n",
               (int) l_pSourcePath, (int) l_pTargetPath, 0, 0, 0, 0);
  }
  else
  {
    UPGLogMsg ("Copy '%s' to '%s' : ERROR\n",
               (int) l_pSourcePath, (int) l_pTargetPath, 0, 0, 0, 0);
  }

  do
  {
    l_status = UPGCopy (l_pSourcePath, l_pTargetPath, FALSE);

    if (ERROR == l_status)   // UPGCopy  failed ...
    {
      UPGLogMsg ("[ERROR] - Copy '%s' to '%s' : FAILED!  Trial:%d\n",
                 (int) l_pSourcePath,
                 (int) l_pTargetPath,
                 ((UPG_MAX_TRIALS-l_nb_trials)+1), 0, 0, 0);
    }
    else
    {
      UPGLogMsg ("Copy '%s' to '%s' : OK\n",
                 (int) l_pSourcePath, (int) l_pTargetPath, 0, 0, 0, 0);
      break;
    }
      l_nb_trials--;

  }
  while (    (ERROR == l_status)
          && (l_nb_trials > 0));

  return (l_status);
}


//! STATUS UncompressBinWithTriesIfError
//!
//! @param char*       p_Path_bin_file [in]
//! @param char*       p_dest_folder   [in]
//!
//! @return OK or ERROR
STATUS UncompressBinWithTriesIfError (char* p_Path_bin_file, char* p_dest_folder)
{
  STATUS l_stus = ERROR;
  int l_nb_trials = UPG_MAX_TRIALS;

  do
  {
    l_stus = UnCompress (p_Path_bin_file, p_dest_folder);

    if (OK == l_stus)
    {
      UPGLogMsg ("Copy '%s' to '%s' : OK\n",
               (int) p_Path_bin_file, (int) p_dest_folder, 0, 0, 0, 0);
    }
    else
    {
      // UnCompress failed ...
      UPGLogMsg ("[ERROR] - UnCompress '%s' to '%s' : FAILED!  Trial:%d\n",
               (int) p_Path_bin_file,
               (int) p_dest_folder,
               ((UPG_MAX_TRIALS-l_nb_trials)+1), 0, 0, 0);
    }
    l_nb_trials--;

  }
  while (    (ERROR == l_stus)
          && (l_nb_trials > 0));

  return (l_stus);
}



STATUS FormatPartNavigation (char *p_pSourceDrive, char *p_pTargetDrive)
{
  STATUS l_script_status = OK;
  BOOL   l_bKeepDBFiles  = FALSE;
  char l_pSourcePath[MAX_PATH_LENGTH];
  char l_pTargetPath[MAX_PATH_LENGTH];
  char l_pSourceVersion[MAX_VERSION_LENGTH];
  char l_pTargetVersion[MAX_VERSION_LENGTH];


  UPGLogMsg ("FormatPartNavigation +, Time = %ld\n", tickGet(), 0, 0, 0, 0, 0);

  //Compare DB_DWN source files with taget files
  //Check the version of DB_DWN
  l_bKeepDBFiles = CheckAndSaveFileIfNeeded (UPG_DB_DWNL_PPC, p_pSourceDrive, p_pTargetDrive);

  if (ERROR == Format (p_pTargetDrive))
  {
    UPGLogMsg ("Cannot format drive 's'\n", (int)p_pTargetDrive, 0, 0, 0, 0, 0);
    l_script_status = ERROR;
  }
  else
  {
    //Keep key files on product
    UPGLogMsg ("Restore key files to '%s'\n", (int)p_pTargetDrive, 0, 0, 0, 0, 0);
    if (TRUE == l_bKeepDBFiles)
    {
      strcpy (l_pSourcePath, TEMP_DB_ZONE);

      CopyFileFromSrcDriveToTargDrive (UPG_DB_DWNL_PPC,     UPG_DB_DWNL_PPC,     l_pSourcePath, p_pTargetDrive);
      CopyFileFromSrcDriveToTargDrive (UPG_DB_DWNL_PPC_INF, UPG_DB_DWNL_PPC_INF, l_pSourcePath, p_pTargetDrive);

      // Remove TEMP_DB_ZONE
      xdelete (TEMP_DB_ZONE);
      UPGMKDelete (TEMP_DB_ZONE);
    }
    else
    {
      CopyFileFromSrcDriveToTargDrive (UPG_DB_DWNL_PPC,     UPG_DB_DWNL_PPC,     p_pSourceDrive, p_pTargetDrive);
      CopyFileFromSrcDriveToTargDrive (UPG_DB_DWNL_PPC_INF, UPG_DB_DWNL_PPC_INF, p_pSourceDrive, p_pTargetDrive);
    }
  }

  UPGLogMsg ("FormatPartNavigation -, Time = %ld\n", tickGet(), 0, 0, 0, 0, 0);

  return (l_script_status);
}

//! STATUS PrepareFilesListToCopy
//!
//! @param char            *p_pSourceDrive  [in]
//! @param char            *p_pTargetDrive  [in]
//! @param char            *p_pCID          [in]
//! @param t_NavUpgData   **p_pList         [out]
//! @param int             *l_iCountList    [out]
//! @param unsigned long   *l_lTotalSize    [out]
//!
//! @return OK or ERROR
STATUS PrepareFilesListToCopy (char            *p_pSourceDrive,
                               char            *p_pTargetDrive,
                               char            *p_pCID,
                               t_NavUpgData   **p_pList,
                               int             *p_piCountList,
                               unsigned long   *p_plTotalSize)
{
  STATUS  l_script_status = OK;
  int     l_iCountFilesforCID = 0;
  char    l_pPath[MAX_PATH_LENGTH];
  char    l_pSearchResult[MAX_PATH_LENGTH];
  int     l_j;

  if (    (NULL == p_pSourceDrive)
       || (NULL == p_pTargetDrive)
       || (NULL == p_pCID)
       || (NULL == p_pList)
       || (NULL == p_piCountList)
       || (NULL == p_plTotalSize)
     )
  {
    UPGLogMsg ("PrepareFilesListToCopy: NULL pointer in parameters !!!!\n", 0, 0, 0, 0, 0, 0);
    return (ERROR);
  }

  //Look for files
  strcpy (l_pPath, p_pSourceDrive);
  strcat (l_pPath, "/MAPPE/");
  strcat (l_pPath, p_pCID);
  strcat (l_pPath, "/");

  UPGLogMsg ("PrepareFilesListToCopy: Search for '%s'\n", l_pPath, 0, 0, 0, 0, 0);
  UPGLogMsg ("PrepareFilesListToCopy: File search in progress... + %d\n", tickGet(), 0, 0, 0, 0, 0);
  l_iCountFilesforCID = UPGSearchFile ("*.*", l_pPath, NULL, NULL);
  UPGLogMsg ("PrepareFilesListToCopy: File search ended...       - %d\n", tickGet(), 0, 0, 0, 0, 0);

  //Allocate list
  if (0 == l_iCountFilesforCID)
  {
    UPGLogMsg ("PrepareFilesListToCopy: 0.files found\n", 0, 0, 0, 0, 0, 0);
    l_script_status = ERROR;
  }

  if (OK == l_script_status)
  {
    for (l_j=0; l_j<l_iCountFilesforCID; l_j++)
    {
      if (ERROR == UPGGetSearchResult (l_j, l_pSearchResult))
      {
        UPGLogMsg ("UPGGetSearchResult error\n", 0, 0, 0, 0, 0, 0);
        l_script_status = ERROR;
        break;
      }

      if (OK == l_script_status)
      {
        *p_pList = (t_NavUpgData*) realloc (*p_pList,
                                            sizeof (t_NavUpgData) * (*p_piCountList+1) );

        //Fill list
        if (NULL != *p_pList)
        {
          strcpy ( (*p_pList)[*p_piCountList].m_pFileName, l_pSearchResult);

          if (OK == ComputeNavTargetPath ((*p_pList)[*p_piCountList].m_pTargetPath,
                                             l_pSearchResult,
                                             p_pCID,
                                             p_pSourceDrive,
                                             p_pTargetDrive)
             )
          {
            if (ERROR == GetFileSize (l_pSearchResult, &((*p_pList)[*p_piCountList].m_iSize)))
            {
              UPGLogMsg ("UPGGetSearchResult error\n", 0, 0, 0, 0, 0, 0);
              l_script_status = ERROR;
              break;
            }

            *p_plTotalSize += (unsigned long) (*p_pList)[*p_piCountList].m_iSize;

            strcpy ((*p_pList)[*p_piCountList].m_pCID, p_pCID);

            (*p_piCountList)++;
          }
        }
        else
        {
          UPGLogMsg ("PrepareFilesListToCopy: p_pList Allocation error\n", 0, 0, 0, 0, 0, 0);
          l_script_status = ERROR;
          break;
        }
      }
    }
  }

  return (l_script_status);
}



//! void UPGSizeManagement
//!
//! @param char            *p_pSourceDrive     [in]
//! @param char            *p_pTargetDrive     [in]
//! @param unsigned long    p_size_to_install  [in]
//! @param BOOL            *p_pbUpgRequired    [out]
//! @param t_TYPE_DEVICE    p_MediaInfoType    [in]
//!
//! @return OK, ERROR, ESCAPE
int UPGSizeManagement ( char          *p_pSourceDrive,
                        char          *p_pTargetDrive,
                        unsigned long  p_size_to_install,
                        BOOL          *p_pbUpgRequired,
                        t_TYPE_DEVICE  p_MediaInfoType)
{
  int             l_script_status   = OK;
  BOOL            l_continue        = TRUE;
  unsigned long   l_free_space_inMB = 0;
  int             l_FoundCountries  = 0;
  t_UPGCIDSize   *l_pCountries      = NULL;
  t_DVD_Content   l_reduced_DVD_Content;
  char            l_user_country_name_choice[MAX_COUNTRY_NAME_LENGTH];
  char            l_user_country_CID_choice[MAX_CID_LENGTH];
  BOOL            l_quit_upg        = FALSE;
  t_CID_List     *l_CID_real_List   = NULL;
  int             l_i               = 0;
  int             l_nbCountries     = 0;
  BOOL            l_EraseCountry    = FALSE;
  unsigned long   l_dummy_size;
  char            l_sDummySize[150];
  char            l_sPath[150];

  *p_pbUpgRequired = FALSE;

  while (TRUE == l_continue)
  {
    l_script_status = UPGGetFreeSpaceOnPartition (p_pTargetDrive, &l_free_space_inMB);
    if (OK == l_script_status)
    {
      l_free_space_inMB = (l_free_space_inMB * FREE_SPACE_RATIO) / 100;

      UPGLogMsg ("UPGSizeManagement(): %d %% of l_free_space = %ld Mb\n", FREE_SPACE_RATIO, l_free_space_inMB, 0, 0, 0, 0);

      if (p_size_to_install >= l_free_space_inMB)
      {
        l_script_status = UPGGetHddCountries (&l_FoundCountries, &l_pCountries);

        if (OK == l_script_status)
        {
          strcpy (l_reduced_DVD_Content.m_DVD_title, "RNEG");
          l_reduced_DVD_Content.m_DVD_CID_number = l_FoundCountries;

          // Fabriquer la liste des CID du HDD
          l_CID_real_List = (t_CID_List*) malloc (sizeof(t_CID_List) * l_FoundCountries);
          if (NULL != l_CID_real_List)
          {
            l_nbCountries = 0;
            for (l_i=0; l_i<l_FoundCountries; l_i++)
            {
              /*    To delete into next version (>1.1.22) if works
              strcpy (l_sPath,"/SDH/MAPPE/");
              strcat (l_sPath,l_pCountries[l_i].pCID);
              strcat (l_sPath,"/CD_VER.NAV.INF");
              UPGLogMsg ("GetCIDSize ('%s')\n", l_sPath, 0, 0, 0, 0, 0);
              GetCIDSize (l_sPath, l_sDummySize,&l_dummy_size);
              l_pCountries[l_i].CIDSize = l_dummy_size;
              UPGLogMsg ("UPGSizeManagement():l_pCountries[%d] = %ld bytes\n",
                l_i, l_pCountries[l_i].CIDSize, 0, 0, 0, 0);

*/

              //"CD_VER_NAV.INF" and "{CID}_EPC.BIN.inf"
              if (OK == l_script_status)
              {

                if (ERROR == GetCorrectCountry_Size ("/SDH",
                                                     l_pCountries[l_i].pCID,
                                                     &l_dummy_size))
                {
                  UPGLogMsg ("[ERROR] - UPGSizeManagement() :  GetCorrectCountry_Size() Failed for '/SDH'\n",
                             0, 0, 0, 0, 0, 0);
                }
                else
                {
                  //we are doing this "l_dummy_size-1" because of "+1" we are doing in GetCorrectCountry_Size function
                  l_pCountries[l_i].CIDSize = (l_dummy_size-1) * (1024*1024);
                  UPGLogMsg ("CDCheckSpaceAndConfirmUpgrade:l_pCountries[%d] = %ld bytes\n",
                l_i, l_pCountries[l_i].CIDSize, 0, 0, 0, 0);
                }

              }


              if (l_pCountries[l_i].CIDSize>0)
              {
                strcpy (l_CID_real_List[l_nbCountries].m_pCID, l_pCountries[l_i].pCID);
                strcpy (l_CID_real_List[l_nbCountries].m_CID_name, l_pCountries[l_i].CountryName);
                l_CID_real_List[l_nbCountries].m_cid_size = l_pCountries[l_i].CIDSize;
                l_nbCountries ++;
              }
              else
              {
                UPGLogMsg ("WARNING UPGGetHddCountries return an empty country !!!!\n", 0, 0, 0, 0, 0, 0);
              }
            }
            l_reduced_DVD_Content.m_CID_list = l_CID_real_List;
            l_reduced_DVD_Content.m_DVD_CID_number = l_nbCountries;

            // Ask to Erase a country
            l_user_country_name_choice[0] = '\0';
            l_user_country_CID_choice[0] = '\0';
            l_quit_upg = TRUE;
            ShowNavigationErasingCountryScreen (&l_reduced_DVD_Content,
                                                 l_user_country_name_choice,
                                                 l_user_country_CID_choice,
                                                &l_quit_upg,
                                                 p_size_to_install-l_free_space_inMB);
            UPGLogMsg ("l_user_country_choice = %s\n", l_user_country_name_choice, 0, 0, 0, 0, 0);
            UPGLogMsg ("l_user_country_choice = %s\n", l_user_country_CID_choice, 0, 0, 0, 0, 0);
            UPGLogMsg ("ShowNavigationEraseCountryScreen done!\n", 0, 0, 0, 0, 0, 0);
            UPGLogMsg ("l_quit_upg = %d\n", l_quit_upg, 0, 0, 0, 0, 0);

            if (FALSE == FileExists(p_pSourceDrive))
            {
              l_quit_upg       = TRUE;
              *p_pbUpgRequired = FALSE;
              l_continue       = FALSE;
              l_script_status  = ERROR;
            }
            else
            {
              if (FALSE == l_quit_upg)
              {
                l_EraseCountry = FALSE;
                ShowNavigationExtendedQuestionScreen (MMI_UPG_DISPLAY_ASK_MOTOR_SUPPRESS_THE_CARD,
                                                      l_user_country_name_choice,
                                                      &l_EraseCountry);
                if (FALSE == FileExists(p_pSourceDrive))
                {
                  l_quit_upg       = TRUE;
                  *p_pbUpgRequired = FALSE;
                  l_continue       = FALSE;
                  l_script_status  = ERROR;
                }
                else
                {
                  if (TRUE == l_EraseCountry)
                  {
                    UPGLogMsg ("=== UPGXdeleteCountry ('%s', '%s') ===\n", p_pTargetDrive, l_user_country_CID_choice, 0, 0, 0, 0);
                    UPGStopGuidance ();
                    UPGXdeleteCountry (p_pTargetDrive, l_user_country_CID_choice);
                    UPGCheckCarto ();
                    if (ERROR == UPGInformNavForNewMapAndWaitAck (p_MediaInfoType))
                    {
                      UPGLogMsg ("ERROR on UPGInformNavForNewMapAndWaitAck\n", 0, 0, 0, 0, 0, 0);
                      l_script_status = ERROR;
                    }
                  }
                  else
                  {
                    *p_pbUpgRequired = FALSE;
                    l_continue       = TRUE;
                  }
                }
              }
              else
              {
                // End erase, Cancel by user
                *p_pbUpgRequired = FALSE;
                l_continue       = FALSE;
                l_script_status  = ESCAPE;
              }
            }

            free (l_CID_real_List);
            l_CID_real_List = NULL;
          }
          else
          {
            // End Size management, malloc Error
            *p_pbUpgRequired = FALSE;
            l_continue       = FALSE;
            l_script_status  = ERROR;
          }
        }
        else
        {
          // End Size management, UPGGetHddCountries Error
          *p_pbUpgRequired = FALSE;
          l_continue       = FALSE;
          l_script_status  = ERROR;
        }
      }
      else
      {
        // p_size_to_install < l_free_space_inMB
        *p_pbUpgRequired  = TRUE;
        l_continue        = FALSE;
        l_script_status   = OK;
      }
    }
    else
    {
      // End Size management, UPGGetFreeSpaceOnPartition Error
      *p_pbUpgRequired = FALSE;
      l_continue       = FALSE;
      l_script_status  = ERROR;
    }
  }

  return (l_script_status);
}



//! void UPGXdeleteCountry
//!
//! @param char *p_pTargetDrive             [in]
//! @param char *p_user_country_CID_choice  [in]
//!
//! @return Nothing
//!
void UPGXdeleteCountry (char *p_pTargetDrive,
                        char *p_user_country_CID_choice)
{
  char l_dummy_path[MAX_PATH_LENGTH];
  char l_string[MAX_STRING_LEN];

  UPGLogMsg ("UPGXdeleteCountry '%s' + (%ld ticks)\n", p_user_country_CID_choice, tickGet(), 0, 0, 0, 0);

  ShowNavigationBargraph();
  taskDelay (100);
  // "PREPARATION" is langDB text...
  StepBargraph ("PREPARATION_TEXT", 0);

  strcpy (l_dummy_path, p_pTargetDrive);
  strcat (l_dummy_path, "/MAPPE/");
  strcat (l_dummy_path, p_user_country_CID_choice);
  strcat (l_dummy_path, "/");
  xdelete (l_dummy_path);
  UPGMKDelete (l_dummy_path);

  strcpy (l_string, "XDELETE of CID !\n");
  strcat (l_string, p_user_country_CID_choice);
  strcat (l_string, "\n");
  AddUPGHistoryCommentary (l_string);

  UPGLogMsg ("UPGXdeleteCountry '%s' - (%ld ticks)\n", p_user_country_CID_choice, tickGet(), 0, 0, 0, 0);
}



//! STATUS UPGInformNavForNewMapAndWaitAck
//!
//! @param t_TYPE_DEVICE  l_MediaInfoType [in]
//!
//! @return OK or ERROR
//!
STATUS UPGInformNavForNewMapAndWaitAck (t_TYPE_DEVICE  l_MediaInfoType)
{
  STATUS  l_Status = OK;

  UPGLogMsg ("Script UPGInformNavForNewMapAndWaitAck +, Time = %ld\n", tickGet(), 0, 0, 0, 0, 0);

  // Send event if new CID
  UPGLogMsg ("SendMessageMMINavCDLoaded Media InfoType %d\n", (int) l_MediaInfoType, 0, 0, 0, 0, 0);
  if (ERROR == SendMessageMMINavCDLoaded ((int) l_MediaInfoType))
  {
    UPGLogMsg ("SendMessageMMINavCDLoaded error\n", 0, 0, 0, 0, 0, 0);
    l_Status = ERROR;
  }
  else
  {
    UPGLogMsg ("SendMessageMMINavCDLoaded OK \n", 0, 0, 0, 0, 0, 0);
    //Call builtin waiting for NAV ACK
    UPGLogMsg ("Waiting  for 'CIDLoaded'\n", 0, 0, 0, 0, 0, 0);
    GetMessageMMINavCIDLoaded();
    UPGLogMsg ("GetMessageMMINavCIDLoaded OK \n", 0, 0, 0, 0, 0, 0);
  }

  UPGLogMsg ("Script UPGInformNavForNewMapAndWaitAck -, Time = %ld\n", tickGet(), 0, 0, 0, 0, 0);

  return (l_Status);
}


BOOL UPGCheckIfVersionIsNewerThan (char *p_pSourcePath, char *p_pTargetPath)
{
  BOOL    l_bIsVersionNewer = TRUE;
  STATUS  l_status          = OK;
  char    l_pSourcePath[MAX_PATH_LENGTH];
  char    l_pTargetPath[MAX_PATH_LENGTH];
  char    l_pDummyPath [MAX_PATH_LENGTH];

  char    l_pSourceVersion[MAX_VERSION_LENGTH];
  char    l_pTargetVersion[MAX_VERSION_LENGTH];

  strcpy (l_pDummyPath, p_pSourcePath);
  strcat (l_pDummyPath, ".inf");
  if (ERROR == GetNavigationValue (l_pDummyPath, "VERSION", l_pSourceVersion))
  {
    if (ERROR == GetNavigationValue (l_pDummyPath, "VER", l_pSourceVersion))
    {
      l_status = ERROR;
      UPGLogMsg ("UPGCheckIfVersionIsNewerThan: GetNavigationValue (VERSION) error on '%s'\n",
                  (int) l_pDummyPath, 0, 0, 0, 0, 0);
    }
  }

  if (OK == l_status)
  {
    UPGLogMsg ("UPGCheckIfVersionIsNewerThan : Source : Version of '%s' is %s\n",
                l_pDummyPath, l_pSourceVersion, 0, 0, 0, 0);
  }

  if (OK == l_status)
  {
    strcpy (l_pDummyPath, p_pTargetPath);
    strcat (l_pDummyPath, ".inf");
    if (ERROR == GetNavigationValue (l_pDummyPath, "VERSION", l_pTargetVersion))
    {
      if (ERROR == GetNavigationValue (l_pDummyPath, "VER", l_pTargetVersion))
      {
        l_status = ERROR;
        UPGLogMsg ("UPGCheckIfVersionIsNewerThan: Target : GetNavigationValue (VERSION) error on '%s'\n",
                    (int) l_pDummyPath, 0, 0, 0, 0, 0);
      }
    }

    if (OK == l_status)
    {
      UPGLogMsg ("UPGCheckIfVersionIsNewerThan : Target : Version of '%s' is %s\n",
                  l_pDummyPath, l_pTargetVersion, 0, 0, 0, 0);
    }

    if (    (OK == l_status)
          && (atoi (l_pSourceVersion) <= atoi (l_pTargetVersion))
       )
    {
      UPGLogMsg ("UPGCheckIfVersionIsNewerThan : FALSE\n", 0, 0, 0, 0, 0, 0);
      l_bIsVersionNewer = FALSE;
    }
    else
    {
      UPGLogMsg ("UPGCheckIfVersionIsNewerThan : TRUE\n", 0, 0, 0, 0, 0, 0);
    }
  }

  return (l_bIsVersionNewer);
}

STATUS UpgIsGruppoValid(char *p_filename, char* p_filepath, char *p_CID)
{
  STATUS  l_status = OK;
  char    l_pDummyPath [MAX_PATH_LENGTH];
  char    l_pSourceVersion[MAX_VERSION_LENGTH];

  strcpy (l_pDummyPath, p_filepath);
  strcat (l_pDummyPath, p_filename);
    AddUPGHistoryCommentary ("UpgIsGruppoValid+");

  if (ERROR == GetNavigationValue (l_pDummyPath, "VERSION", l_pSourceVersion))
  {
    if (ERROR == GetNavigationValue (l_pDummyPath, "VER", l_pSourceVersion))
    {
      l_status = ERROR;
      UPGLogMsg ("UpgIsGruppoValid: GetNavigationValue (VERSION) error on '%s'\n",
                  (int) l_pDummyPath, 0, 0, 0, 0, 0);
    }
  }

  if(0 != strcmp(l_pSourceVersion,p_CID))
  {
    l_status = ERROR;
        AddUPGHistoryCommentary ("Versions differentes");
  }
  else
  {
    AddUPGHistoryCommentary ("Versions identiques");
  }

  AddUPGHistoryCommentary ("UpgIsGruppoValid-");

  return(l_status);
}




//Do not treat p_quit_asked parameter now.Somethinbg to understant first in Main function
int CanUpgContinue (char* p_src_drive, int p_quit_asked, STATUS l_progress_status)
{
  /*if ((TRUE == FileExists(p_src_drive)) &&
    (FALSE == p_quit_asked) &&
    (OK == l_progress_status))*/
  if ((TRUE == FileExists(p_src_drive)) && (OK == l_progress_status))
  {
    return (TRUE);
  }
  return (FALSE);
}





//Since carto unique for Europe the correct size of carto depends on
//  CD_VERS_NAV.INF file and {CID}_EPC.BIN.inf file
//  During version 1.1.23 of the map file the "{CID}_EPC.BIN.inf" file is also
//  copy on device during upgrade process.
//
//  Country size is the size contained in CD_VERS_NAV.INF + size in {CID}_EPC.BIN.inf
//  if the current installed country doesn't have "{CID}_EPC.BIN.inf" file,
//this function will return only the size in "CD_VERS_NAV.INF" file
//
//    p_path_dir refers to root dir which means "/SDH"  or "/bd0/Data"
STATUS GetCorrectCountry_Size(char* p_path_dir, char* p_cid, unsigned long *p_total_size)
{


  STATUS          l_status   = ERROR;

  char            l_pPathInf[MAX_PATH_LENGTH];
  char            l_pCD_CID_SIZE[MAX_INF_LINE_LENGTH];
  char            l_pCD_CID_EPC_SIZE[MAX_INF_LINE_LENGTH];

  unsigned long   l_CID_size = 0;
  unsigned long   l_CID_EPC_size = 0;
  unsigned long   l_total_size = 0;
/*

  char l_pPath[MAX_PATH_LENGTH];
  char l_pPathInf[MAX_PATH_LENGTH];
  char l_pCD_NAME[MAX_INF_LINE_LENGTH];
  char l_pCD_CID[MAX_INF_LINE_LENGTH];
  char l_pCD_PROVIDER[MAX_INF_LINE_LENGTH];
  char l_pQuestionText[MAX_QUESTION_LENGTH];

  int             l_FoundCountries  = 0;
  t_UPGCIDSize   *l_pCountries      = NULL;
  unsigned long   l_free_space_inMB = 0;
  BOOL            l_Continue        = TRUE;
  int             l_count           = 0;
  int             l_return          = 0;
  unsigned long   l_supposed_new_space_after_remove = 0;
  unsigned long   l_dummy_size;
  char            l_sDummySize[150];
  char            l_sPath[150];
*/

  if((NULL == p_path_dir) || (NULL == p_cid))
  {
    UPGLogMsg ("[ERROR] - GetCorrectCountry_Size() : Some parameter are NULL\n",
               0, 0, 0, 0, 0, 0);
    return (l_status);
  }

  UPGLogMsg ("GetCorrectCountry_Size() : Compute correct country Size on '%s' \n",
             p_cid, 0, 0, 0, 0, 0);
  l_status = OK;

  //CD_VER_NAV.INF
  if (OK == l_status)
  {
    strcpy (l_pPathInf, p_path_dir);
    strcat (l_pPathInf, UPG_PATH_DELIMITER);
    strcat (l_pPathInf, UPG_MAPPE_DIR);
    strcat (l_pPathInf, p_cid);
    strcat (l_pPathInf, KEY_FILE_NAV);
    strcat (l_pPathInf, UPG_INF_XTENSION);

    if (ERROR == GetCIDSize (l_pPathInf, l_pCD_CID_SIZE, &l_CID_size))
    {
      UPGLogMsg ("[ERROR] - GetCorrectCountry_Size() : GetCIDSize Failed on %s\n",
                  l_pPathInf, 0, 0, 0, 0, 0);
      l_status = ERROR;
    }
    else
    {
      l_CID_size = (l_CID_size/(1024*1024))+1;
      l_total_size += l_CID_size;
      UPGLogMsg ("GetCorrectCountry_Size():  CID Size = %d Mb\n",
                 l_CID_size, 0, 0, 0, 0, 0);
    }
  }


  //{CID}_EPC.BIN.inf
  if (OK == l_status)
  {
    strcpy (l_pPathInf, p_path_dir);
    strcat (l_pPathInf, UPG_PATH_DELIMITER);
    strcat (l_pPathInf, UPG_MAPPE_DIR);
    strcat (l_pPathInf, p_cid);
    strcat (l_pPathInf, UPG_PATH_DELIMITER);
    strcat (l_pPathInf, p_cid);
    strcat (l_pPathInf, UPG_EPC_INF_SURFIXE);

    //check first if the file exist
    if (TRUE == FileExists(l_pPathInf))
    {
      if (ERROR == GetCID_EPCSize (l_pPathInf, l_pCD_CID_EPC_SIZE, &l_CID_EPC_size))
      {
        UPGLogMsg ("[ERROR] - GetCorrectCountry_Size() : GetCID_EPCSize Failed on %s\n",
                   l_pPathInf, 0, 0, 0, 0, 0);
        l_status = ERROR;
      }
      else
      {
        l_CID_EPC_size = (l_CID_EPC_size/(1024*1024))+1;
        l_total_size += l_CID_EPC_size;
        UPGLogMsg ("GetCorrectCountry_Size():  CID EPC Size = %d Mb\n",
                   l_CID_EPC_size, 0, 0, 0, 0, 0);
        UPGLogMsg ("CDCheckSpaceAndConfirmUpgrade:  l_size_of_EPC_to_install = %d Mb\n", l_CID_EPC_size, 0, 0, 0, 0, 0);
      }
    }
  }

  //save the size
  *p_total_size = l_total_size;
  UPGLogMsg ("GetCorrectCountry_Size():  Total Size = %d Mb\n",
             l_total_size, 0, 0, 0, 0, 0);

  return (l_status);

}





STATUS      LogBuiltinsAndMapVersion (char*   p_drive)
{

  char      l_pBuiltinsFile[MAX_PATH_LENGTH];
  char      l_pMapFile[MAX_PATH_LENGTH];

  char      l_BuiltinsVersion[MAX_VERSION_LENGTH];
  char      l_MapVersion[MAX_VERSION_LENGTH];

  STATUS    l_status = ERROR;

  if (NULL == p_drive)
  {
    UPGLogMsg ( "[ERROR] - LogBuiltinsAndMapVersion() Parameter should not be NULL\n", 0,0,0,0,0,0);
    return (l_status);
  }

  l_status = OK;
  //bild path
  strcpy ( l_pBuiltinsFile, p_drive );
  strcat ( l_pBuiltinsFile, BUILTINS_FILE_NORMAL );

  strcpy ( l_pMapFile, p_drive );
  strcat ( l_pMapFile, UPG_MAP );

  UPGLogMsg ( "******************************************************\n", 0,0,0,0,0,0);
  if (OK == GetInfValue(l_pBuiltinsFile, UPG_KEY_WORD, l_BuiltinsVersion))
  {
    UPGLogMsg ( "Builtins Version :  %s\n", (int)l_BuiltinsVersion, 0, 0, 0, 0, 0);
  }
  else
  {
    l_status = ERROR;
    UPGLogMsg ("[ERROR] - LogBuiltinsAndMapVersion() - GetInfValue failed on '%s' \n",
               (int)l_pBuiltinsFile,0,0,0,0,0);
  }

  if (OK == GetInfValue(l_pMapFile, UPG_KEY_WORD, l_MapVersion))
  {
    UPGLogMsg ("Map Version :  %s\n", (int)l_MapVersion, 0, 0, 0, 0, 0);
  }
  else
  {
    l_status = ERROR;
    UPGLogMsg ("[ERROR] - LogBuiltinsAndMapVersion() - GetInfValue failed on '%s' \n",
               (int)l_pMapFile,0,0,0,0,0);
  }
  UPGLogMsg ( "******************************************************\n", 0,0,0,0,0,0);

  return (l_status);
}


